# Auto generated configuration file
# using:
# Revision: 1.341
# Source: /cvs/CMSSW/CMSSW/Configuration/PyReleaseValidation/python/ConfigBuilder.py,v
# with command line options: skim -s SKIM:LogError+EXOHSCP --data --conditions FT_R_44_V11::All --python_filenam skim_BTag.py --magField AutoFromDBCurrent --no_exec
# modified to reduce outside dependencies - this probably doesn't do anything useful anymore
# hopefully its still useful as a test

import FWCore.ParameterSet.Config as cms

process = cms.Process('SKIM')

# import of standard configurations
#process.load('Configuration.StandardSequences.Services_cff')
#process.load('SimGeneral.HepPDTESSource.pythiapdt_cfi')
#process.load('FWCore.MessageService.MessageLogger_cfi')
#process.load('Configuration.EventContent.EventContent_cff')
#process.load('Configuration.StandardSequences.GeometryDB_cff')
#process.load('Configuration.StandardSequences.MagneticField_AutoFromDBCurrent_cff')
#process.load('Configuration.StandardSequences.Skims_cff')
#process.load('Configuration.StandardSequences.FrontierConditions_GlobalTag_cff')

process.maxEvents = cms.untracked.PSet(
    input = cms.untracked.int32(1)
)

# Input source
process.source = cms.Source("PoolSource",
    secondaryFileNames = cms.untracked.vstring(),
    fileNames = cms.untracked.vstring('file:skim_RECO.root')
)

process.options = cms.untracked.PSet(

)

# Production Info
process.configurationMetadata = cms.untracked.PSet(
    version = cms.untracked.string('$Revision: 1.341 $'),
    annotation = cms.untracked.string('skim nevts:1'),
    name = cms.untracked.string('PyReleaseValidation')
)

# Output definition

# Additional output definition
process.SKIMStreamEXOHSCP = cms.OutputModule("PoolOutputModule",
    SelectEvents = cms.untracked.PSet(
        SelectEvents = cms.vstring('EXOHSCPPath')
    ),
    outputCommands = cms.untracked.vstring('drop *',
        'keep GenEventInfoProduct_generator_*_*',
        'keep L1GlobalTriggerReadoutRecord_*_*_*',
        'keep recoVertexs_offlinePrimaryVertices_*_*',
        'keep recoMuons_muonsSkim_*_*',
        'keep SiStripClusteredmNewDetSetVector_generalTracksSkim_*_*',
        'keep SiPixelClusteredmNewDetSetVector_generalTracksSkim_*_*',
        'keep recoTracks_generalTracksSkim_*_*',
        'keep recoTrackExtras_generalTracksSkim_*_*',
        'keep TrackingRecHitsOwned_generalTracksSkim_*_*',
        'keep *_dt1DRecHits_*_*',
        'keep *_dt4DSegments_*_*',
        'keep *_csc2DRecHits_*_*',
        'keep *_cscSegments_*_*',
        'keep *_rpcRecHits_*_*',
        'keep recoTracks_standAloneMuons_*_*',
        'keep recoTrackExtras_standAloneMuons_*_*',
        'keep TrackingRecHitsOwned_standAloneMuons_*_*',
        'keep recoTracks_globalMuons_*_*',
        'keep recoTrackExtras_globalMuons_*_*',
        'keep TrackingRecHitsOwned_globalMuons_*_*',
        'keep EcalRecHitsSorted_reducedHSCPEcalRecHitsEB_*_*',
        'keep EcalRecHitsSorted_reducedHSCPEcalRecHitsEE_*_*',
        'keep HBHERecHitsSorted_reducedHSCPhbhereco__*',
        'keep edmTriggerResults_TriggerResults__*',
        'keep *_hltTriggerSummaryAOD_*_*',
        'keep *_HSCPIsolation01__*',
        'keep *_HSCPIsolation03__*',
        'keep *_HSCPIsolation05__*',
        'keep recoPFJets_ak5PFJets__*',
        'keep recoPFMETs_pfMet__*',
        'keep recoBeamSpot_offlineBeamSpot__*'),
    fileName = cms.untracked.string('EXOHSCP.root'),
    dataset = cms.untracked.PSet(
        filterName = cms.untracked.string('EXOHSCP'),
        dataTier = cms.untracked.string('USER')
    ),
    eventAutoFlushCompressedSize = cms.untracked.int32(5242880)
)
process.SKIMStreamLogError = cms.OutputModule("PoolOutputModule",
    SelectEvents = cms.untracked.PSet(
        SelectEvents = cms.vstring('pathlogerror')
    ),
    outputCommands = (cms.untracked.vstring('drop *', 'drop *', 'keep  FEDRawDataCollection_rawDataCollector_*_*', 'keep  FEDRawDataCollection_source_*_*', 'keep  FEDRawDataCollection_rawDataCollector_*_*', 'keep  FEDRawDataCollection_source_*_*', 'drop *_hlt*_*_*', 'keep *_hltL1GtObjectMap_*_*', 'keep FEDRawDataCollection_hltRawDataRepacker_*_*', 'keep FEDRawDataCollection_hltVirginRawDataRepacker_*_*', 'keep edmTriggerResults_*_*_*', 'keep triggerTriggerEvent_*_*_*', 'keep DetIdedmEDCollection_siStripDigis_*_*', 'keep DetIdedmEDCollection_siPixelDigis_*_*', 'keep *_siPixelClusters_*_*', 'keep *_siStripClusters_*_*', 'keep *_dt1DRecHits_*_*', 'keep *_dt4DSegments_*_*', 'keep *_dt1DCosmicRecHits_*_*', 'keep *_dt4DCosmicSegments_*_*', 'keep *_csc2DRecHits_*_*', 'keep *_cscSegments_*_*', 'keep *_rpcRecHits_*_*', 'keep *_hbhereco_*_*', 'keep *_hfreco_*_*', 'keep *_horeco_*_*', 'keep HBHERecHitsSorted_hbherecoMB_*_*', 'keep HORecHitsSorted_horecoMB_*_*', 'keep HFRecHitsSorted_hfrecoMB_*_*', 'keep ZDCDataFramesSorted_*Digis_*_*', 'keep ZDCRecHitsSorted_*_*_*', 'keep *_reducedHcalRecHits_*_*', 'keep *_castorreco_*_*', 'keep *_ecalPreshowerRecHit_*_*', 'keep *_ecalRecHit_*_*', 'keep *_ecalCompactTrigPrim_*_*', 'keep *_ecalTPSkim_*_*', 'keep *_selectDigi_*_*', 'keep EcalRecHitsSorted_reducedEcalRecHitsEE_*_*', 'keep EcalRecHitsSorted_reducedEcalRecHitsEB_*_*', 'keep EcalRecHitsSorted_reducedEcalRecHitsES_*_*', 'keep *_hybridSuperClusters_*_*', 'keep recoSuperClusters_correctedHybridSuperClusters_*_*', 'keep *_multi5x5SuperClusters_*_*', 'keep recoSuperClusters_multi5x5SuperClusters_*_*', 'keep recoSuperClusters_multi5x5SuperClustersWithPreshower_*_*', 'keep recoSuperClusters_correctedMulti5x5SuperClustersWithPreshower_*_*', 'keep recoPreshowerClusters_multi5x5SuperClustersWithPreshower_*_*', 'keep recoPreshowerClusterShapes_multi5x5PreshowerClusterShape_*_*', 'drop recoClusterShapes_*_*_*', 'drop recoBasicClustersToOnerecoClusterShapesAssociation_*_*_*', 'drop recoBasicClusters_multi5x5BasicClusters_multi5x5BarrelBasicClusters_*', 'drop recoSuperClusters_multi5x5SuperClusters_multi5x5BarrelSuperClusters_*', 'keep *_CkfElectronCandidates_*_*', 'keep *_GsfGlobalElectronTest_*_*', 'keep *_electronMergedSeeds_*_*', 'keep recoGsfTracks_electronGsfTracks_*_*', 'keep recoGsfTrackExtras_electronGsfTracks_*_*', 'keep recoTrackExtras_electronGsfTracks_*_*', 'keep TrackingRecHitsOwned_electronGsfTracks_*_*', 'keep recoTracks_generalTracks_*_*', 'keep recoTrackExtras_generalTracks_*_*', 'keep TrackingRecHitsOwned_generalTracks_*_*', 'keep recoTracks_beamhaloTracks_*_*', 'keep recoTrackExtras_beamhaloTracks_*_*', 'keep TrackingRecHitsOwned_beamhaloTracks_*_*', 'keep recoTracks_regionalCosmicTracks_*_*', 'keep recoTrackExtras_regionalCosmicTracks_*_*', 'keep TrackingRecHitsOwned_regionalCosmicTracks_*_*', 'keep recoTracks_rsWithMaterialTracks_*_*', 'keep recoTrackExtras_rsWithMaterialTracks_*_*', 'keep TrackingRecHitsOwned_rsWithMaterialTracks_*_*', 'keep recoTracks_conversionStepTracks_*_*', 'keep recoTrackExtras_conversionStepTracks_*_*', 'keep TrackingRecHitsOwned_conversionStepTracks_*_*', 'keep *_ctfPixelLess_*_*', 'keep *_dedxTruncated40_*_*', 'keep *_dedxDiscrimASmi_*_*', 'keep *_dedxHarmonic2_*_*', 'keep *_trackExtrapolator_*_*', 'keep *_kt4CaloJets_*_*', 'keep *_kt6CaloJets_*_*', 'keep *_ak5CaloJets_*_*', 'keep *_ak7CaloJets_*_*', 'keep *_iterativeCone5CaloJets_*_*', 'keep *_iterativeCone15CaloJets_*_*', 'keep *_kt4PFJets_*_*', 'keep *_kt6PFJets_*_*', 'keep *_ak5PFJets_*_*', 'keep *_ak7PFJets_*_*', 'keep *_iterativeCone5PFJets_*_*', 'keep *_JetPlusTrackZSPCorJetAntiKt5_*_*', 'keep *_ak5TrackJets_*_*', 'keep *_kt4TrackJets_*_*', 'keep recoRecoChargedRefCandidates_trackRefsForJets_*_*', 'keep *_caloTowers_*_*', 'keep *_towerMaker_*_*', 'keep *_CastorTowerReco_*_*', 'keep *_ic5JetTracksAssociatorAtVertex_*_*', 'keep *_iterativeCone5JetTracksAssociatorAtVertex_*_*', 'keep *_iterativeCone5JetTracksAssociatorAtCaloFace_*_*', 'keep *_iterativeCone5JetExtender_*_*', 'keep *_kt4JetTracksAssociatorAtVertex_*_*', 'keep *_kt4JetTracksAssociatorAtCaloFace_*_*', 'keep *_kt4JetExtender_*_*', 'keep *_ak5JetTracksAssociatorAtVertex_*_*', 'keep *_ak5JetTracksAssociatorAtCaloFace_*_*', 'keep *_ak5JetExtender_*_*', 'keep *_ak7JetTracksAssociatorAtVertex_*_*', 'keep *_ak7JetTracksAssociatorAtCaloFace_*_*', 'keep *_ak7JetExtender_*_*', 'keep *_ak5JetID_*_*', 'keep *_ak7JetID_*_*', 'keep *_ic5JetID_*_*', 'keep *_kt4JetID_*_*', 'keep *_kt6JetID_*_*', 'keep *_ak7BasicJets_*_*', 'keep *_ak7CastorJetID_*_*', 'keep recoCaloMETs_met_*_*', 'keep recoCaloMETs_metNoHF_*_*', 'keep recoCaloMETs_metHO_*_*', 'keep recoCaloMETs_corMetGlobalMuons_*_*', 'keep recoCaloMETs_metNoHFHO_*_*', 'keep recoCaloMETs_metOptHO_*_*', 'keep recoCaloMETs_metOpt_*_*', 'keep recoCaloMETs_metOptNoHFHO_*_*', 'keep recoCaloMETs_metOptNoHF_*_*', 'keep recoMETs_htMetAK5_*_*', 'keep recoMETs_htMetAK7_*_*', 'keep recoMETs_htMetIC5_*_*', 'keep recoMETs_htMetKT4_*_*', 'keep recoMETs_htMetKT6_*_*', 'keep recoMETs_tcMet_*_*', 'keep recoMETs_tcMetWithPFclusters_*_*', 'keep recoPFMETs_pfMet_*_*', 'keep recoMuonMETCorrectionDataedmValueMap_muonMETValueMapProducer_*_*', 'keep recoMuonMETCorrectionDataedmValueMap_muonTCMETValueMapProducer_*_*', 'keep recoHcalNoiseRBXs_hcalnoise_*_*', 'keep HcalNoiseSummary_hcalnoise_*_*', 'keep *HaloData_*_*_*', 'keep *BeamHaloSummary_BeamHaloSummary_*_*', 'keep *_MuonSeed_*_*', 'keep *_ancientMuonSeed_*_*', 'keep *_mergedStandAloneMuonSeeds_*_*', 'keep TrackingRecHitsOwned_globalMuons_*_*', 'keep TrackingRecHitsOwned_tevMuons_*_*', 'keep recoCaloMuons_calomuons_*_*', 'keep *_CosmicMuonSeed_*_*', 'keep recoTrackExtras_cosmicMuons_*_*', 'keep TrackingRecHitsOwned_cosmicMuons_*_*', 'keep recoTrackExtras_globalCosmicMuons_*_*', 'keep TrackingRecHitsOwned_globalCosmicMuons_*_*', 'keep recoTrackExtras_cosmicMuons1Leg_*_*', 'keep TrackingRecHitsOwned_cosmicMuons1Leg_*_*', 'keep recoTrackExtras_globalCosmicMuons1Leg_*_*', 'keep TrackingRecHitsOwned_globalCosmicMuons1Leg_*_*', 'keep recoTracks_cosmicsVetoTracks_*_*', 'keep *_SETMuonSeed_*_*', 'keep recoTracks_standAloneSETMuons_*_*', 'keep recoTrackExtras_standAloneSETMuons_*_*', 'keep TrackingRecHitsOwned_standAloneSETMuons_*_*', 'keep recoTracks_globalSETMuons_*_*', 'keep recoTrackExtras_globalSETMuons_*_*', 'keep TrackingRecHitsOwned_globalSETMuons_*_*', 'keep recoMuons_muonsWithSET_*_*', 'keep *_muons_*_*', 'keep *_*_muons_*', 'drop *_muons_muons1stStep2muonsMap_*', 'keep recoTracks_standAloneMuons_*_*', 'keep recoTrackExtras_standAloneMuons_*_*', 'keep TrackingRecHitsOwned_standAloneMuons_*_*', 'keep recoTracks_globalMuons_*_*', 'keep recoTrackExtras_globalMuons_*_*', 'keep recoTracks_tevMuons_*_*', 'keep recoTrackExtras_tevMuons_*_*', 'keep recoTracks_generalTracks_*_*', 'keep recoTracksToOnerecoTracksAssociation_tevMuons_*_*', 'keep recoTracks_cosmicMuons_*_*', 'keep recoTracks_globalCosmicMuons_*_*', 'keep recoMuons_muonsFromCosmics_*_*', 'keep recoTracks_cosmicMuons1Leg_*_*', 'keep recoTracks_globalCosmicMuons1Leg_*_*', 'keep recoMuons_muonsFromCosmics1Leg_*_*', 'keep *_muIsoDepositTk_*_*', 'keep *_muIsoDepositCalByAssociatorTowers_*_*', 'keep *_muIsoDepositCalByAssociatorHits_*_*', 'keep *_muIsoDepositJets_*_*', 'keep *_muGlobalIsoDepositCtfTk_*_*', 'keep *_muGlobalIsoDepositCalByAssociatorTowers_*_*', 'keep *_muGlobalIsoDepositCalByAssociatorHits_*_*', 'keep *_muGlobalIsoDepositJets_*_*', 'keep *_impactParameterTagInfos_*_*', 'keep *_trackCountingHighEffBJetTags_*_*', 'keep *_trackCountingHighPurBJetTags_*_*', 'keep *_jetProbabilityBJetTags_*_*', 'keep *_jetBProbabilityBJetTags_*_*', 'keep *_secondaryVertexTagInfos_*_*', 'keep *_ghostTrackVertexTagInfos_*_*', 'keep *_simpleSecondaryVertexBJetTags_*_*', 'keep *_simpleSecondaryVertexHighEffBJetTags_*_*', 'keep *_simpleSecondaryVertexHighPurBJetTags_*_*', 'keep *_combinedSecondaryVertexBJetTags_*_*', 'keep *_combinedSecondaryVertexMVABJetTags_*_*', 'keep *_ghostTrackBJetTags_*_*', 'keep *_btagSoftElectrons_*_*', 'keep *_softElectronCands_*_*', 'keep *_softPFElectrons_*_*', 'keep *_softElectronTagInfos_*_*', 'keep *_softElectronBJetTags_*_*', 'keep *_softElectronByIP3dBJetTags_*_*', 'keep *_softElectronByPtBJetTags_*_*', 'keep *_softMuonTagInfos_*_*', 'keep *_softMuonBJetTags_*_*', 'keep *_softMuonByIP3dBJetTags_*_*', 'keep *_softMuonByPtBJetTags_*_*', 'keep *_combinedMVABJetTags_*_*', 'keep *_ak5PFJetsRecoTauPiZeros_*_*', 'keep *_hpsPFTauProducer*_*_*', 'keep *_hpsPFTauDiscrimination*_*_*', 'keep *_shrinkingConePFTauProducer*_*_*', 'keep *_shrinkingConePFTauDiscrimination*_*_*', 'keep *_hpsTancTaus_*_*', 'keep *_hpsTancTausDiscrimination*_*_*', 'keep *_TCTauJetPlusTrackZSPCorJetAntiKt5_*_*', 'keep *_caloRecoTauTagInfoProducer_*_*', 'keep recoCaloTaus_caloRecoTauProducer*_*_*', 'keep *_caloRecoTauDiscrimination*_*_*', 'keep  *_offlinePrimaryVertices__*', 'keep  *_offlinePrimaryVerticesWithBS_*_*', 'keep  *_offlinePrimaryVerticesFromCosmicTracks_*_*', 'keep  *_nuclearInteractionMaker_*_*', 'keep *_generalV0Candidates_*_*', 'keep recoGsfElectronCores_gsfElectronCores_*_*', 'keep recoGsfElectrons_gsfElectrons_*_*', 'keep recoGsfElectronCores_uncleanedOnlyGsfElectronCores_*_*', 'keep recoGsfElectrons_uncleanedOnlyGsfElectrons_*_*', 'keep floatedmValueMap_eidRobustLoose_*_*', 'keep floatedmValueMap_eidRobustTight_*_*', 'keep floatedmValueMap_eidRobustHighEnergy_*_*', 'keep floatedmValueMap_eidLoose_*_*', 'keep floatedmValueMap_eidTight_*_*', 'keep recoPhotons_photons_*_*', 'keep recoPhotonCores_photonCore_*_*', 'keep recoConversions_conversions_*_*', 'drop *_conversions_uncleanedConversions_*', 'keep recoConversions_allConversions_*_*', 'keep recoTracks_ckfOutInTracksFromConversions_*_*', 'keep recoTracks_ckfInOutTracksFromConversions_*_*', 'keep recoTrackExtras_ckfOutInTracksFromConversions_*_*', 'keep recoTrackExtras_ckfInOutTracksFromConversions_*_*', 'keep TrackingRecHitsOwned_ckfOutInTracksFromConversions_*_*', 'keep TrackingRecHitsOwned_ckfInOutTracksFromConversions_*_*', 'keep recoConversions_uncleanedOnlyAllConversions_*_*', 'keep recoTracks_uncleanedOnlyCkfOutInTracksFromConversions_*_*', 'keep recoTracks_uncleanedOnlyCkfInOutTracksFromConversions_*_*')+cms.untracked.vstring('keep recoTrackExtras_uncleanedOnlyCkfOutInTracksFromConversions_*_*', 'keep recoTrackExtras_uncleanedOnlyCkfInOutTracksFromConversions_*_*', 'keep TrackingRecHitsOwned_uncleanedOnlyCkfOutInTracksFromConversions_*_*', 'keep TrackingRecHitsOwned_uncleanedOnlyCkfInOutTracksFromConversions_*_*', 'keep *_PhotonIDProd_*_*', 'keep *_hfRecoEcalCandidate_*_*', 'keep *_hfEMClusters_*_*', 'keep *_pixelTracks_*_*', 'keep *_pixelVertices_*_*', 'drop CaloTowersSorted_towerMakerPF_*_*', 'keep recoPFRecHits_particleFlowClusterECAL_Cleaned_*', 'keep recoPFRecHits_particleFlowClusterHCAL_Cleaned_*', 'keep recoPFRecHits_particleFlowClusterHFEM_Cleaned_*', 'keep recoPFRecHits_particleFlowClusterHFHAD_Cleaned_*', 'keep recoPFRecHits_particleFlowClusterPS_Cleaned_*', 'keep recoPFRecHits_particleFlowRecHitECAL_Cleaned_*', 'keep recoPFRecHits_particleFlowRecHitHCAL_Cleaned_*', 'keep recoPFRecHits_particleFlowRecHitPS_Cleaned_*', 'keep recoPFClusters_particleFlowClusterECAL_*_*', 'keep recoPFClusters_particleFlowClusterHCAL_*_*', 'keep recoPFClusters_particleFlowClusterPS_*_*', 'keep recoPFBlocks_particleFlowBlock_*_*', 'keep recoPFCandidates_particleFlow_*_*', 'keep recoPFDisplacedVertexs_particleFlowDisplacedVertex_*_*', 'keep *_pfElectronTranslator_*_*', 'keep *_pfPhotonTranslator_*_*', 'keep *_particleFlow_electrons_*', 'keep *_particleFlow_photons_*', 'keep *_trackerDrivenElectronSeeds_preid_*', 'keep *_offlineBeamSpot_*_*', 'keep L1GlobalTriggerReadoutRecord_gtDigis_*_*', 'keep *_l1GtRecord_*_*', 'keep *_l1GtTriggerMenuLite_*_*', 'keep *_conditionsInEdm_*_*', 'keep *_l1extraParticles_*_*', 'keep L1MuGMTReadoutCollection_gtDigis_*_*', 'keep L1GctEmCand*_gctDigis_*_*', 'keep L1GctJetCand*_gctDigis_*_*', 'keep L1GctEtHad*_gctDigis_*_*', 'keep L1GctEtMiss*_gctDigis_*_*', 'keep L1GctEtTotal*_gctDigis_*_*', 'keep L1GctHtMiss*_gctDigis_*_*', 'keep L1GctJetCounts*_gctDigis_*_*', 'keep L1GctHFRingEtSums*_gctDigis_*_*', 'keep L1GctHFBitCounts*_gctDigis_*_*', 'keep LumiDetails_lumiProducer_*_*', 'keep LumiSummary_lumiProducer_*_*', 'drop *_hlt*_*_*', 'keep *_hltL1GtObjectMap_*_*', 'keep edmTriggerResults_*_*_*', 'keep triggerTriggerEvent_*_*_*', 'keep L1AcceptBunchCrossings_*_*_*', 'keep L1TriggerScalerss_*_*_*', 'keep Level1TriggerScalerss_*_*_*', 'keep LumiScalerss_*_*_*', 'keep BeamSpotOnlines_*_*_*', 'keep DcsStatuss_*_*_*', 'keep *_logErrorHarvester_*_*', 'drop *_MEtoEDMConverter_*_*', 'drop *_*_*_SKIM')),
    fileName = cms.untracked.string('LogError.root'),
    dataset = cms.untracked.PSet(
        filterName = cms.untracked.string('LogError'),
        dataTier = cms.untracked.string('RAW-RECO')
    ),
    eventAutoFlushCompressedSize = cms.untracked.int32(5242880)
)

# Other statements
#process.GlobalTag.globaltag = 'FT_R_44_V11::All'

# Path and EndPath definitions
process.SKIMStreamEXOHSCPOutPath = cms.EndPath(process.SKIMStreamEXOHSCP)
process.SKIMStreamLogErrorOutPath = cms.EndPath(process.SKIMStreamLogError)

# Schedule definition
#process.schedule = cms.Schedule(process.pathlogerror,process.EXOHSCPPath,process.SKIMStreamEXOHSCPOutPath,process.SKIMStreamLogErrorOutPath)
process.schedule = cms.Schedule(process.SKIMStreamEXOHSCPOutPath,process.SKIMStreamLogErrorOutPath)



#!/usr/bin/env python
"""
_SRMV2Impl_

Implementation of StageOutImpl interface for SRM Version 2

"""
from __future__ import print_function

from builtins import zip, range

import os
import re

from WMCore.Storage.Execute import runCommandWithOutput
from WMCore.Storage.Registry import registerStageOutImpl
from WMCore.Storage.StageOutError import StageOutError
from WMCore.Storage.StageOutImpl import StageOutImpl

_CheckExitCodeOption = True


class SRMV2Impl(StageOutImpl):
    """
    _SRMV2Impl_

    Implement interface for srmcp v2 command

    """
    # also used in unittests
    run = staticmethod(runCommandWithOutput)

    def __init__(self, stagein=False):
        StageOutImpl.__init__(self, stagein)

    def createSourceName(self, protocol, pfn):
        """
        _createSourceName_

        SRM uses file:/// urls

        """
        if pfn.startswith('/'):
            return "file:///%s" % pfn
        elif os.path.isfile(pfn):
            return "file:///%s" % os.path.abspath(pfn)
        else:
            return pfn

    def createOutputDirectory(self, targetPFN):
        """
        _createOutputDirectory_

        SRMV2 does not create directories,
            see http://sdm.lbl.gov/srm-wg/doc/SRM.v2.2.html#_Toc199734394

        """
        targetdir = os.path.dirname(targetPFN)
        if self.stageIn:
            # stage in to local directory - should exist but you never know
            if not os.path.exists(targetdir):
                os.makedirs(targetdir)
            return

        mkdircommand = "srmmkdir -retry_num=0 "
        checkdircmd = "srmls -recursion_depth=0 -retry_num=1 "

        #  // Loop from top level checking existence stop when directory exists
        # // assume first 4 slashes are from srm://host:8443/srm/managerv2?SFN=
        dirs = ["/".join(targetdir.split("/")[0:6 + i]) \
                for i in range(targetdir.count("/") - 4)]
        dirsToCheck = dirs[:]
        dirsToCheck.reverse()
        levelToCreateFrom = len(dirs)
        for count, folder in zip(range(len(dirsToCheck), 0, -1), dirsToCheck):
            try:
                exitCode, output = self.run(checkdircmd + folder)
                levelToCreateFrom = count  # create dirs from here (at least)
                if exitCode:  # did srmls fail to execute properly?
                    raise RuntimeError("ERROR checking directory existence, %s" % str(output))
                if not output.count('SRM_FAILURE'):  # any other codes?
                    break
            except Exception as ex:
                msg = "Warning: Exception while invoking command:\n"
                msg += "%s\n" % checkdircmd + folder
                msg += "Exception: %s\n" % str(ex)
                msg += "Go on anyway..."
                print(msg)

        # // Create needed directory levels from end of previous loop
        # //  to end of directory structure
        for folder in dirs[levelToCreateFrom:]:
            print("Create directory: %s" % folder)
            try:
                exitCode, output = self.run(mkdircommand + folder)
                if exitCode:
                    raise RuntimeError("ERROR creating directory, %s" % str(output))
            except Exception as ex:
                msg = "Warning: Exception while invoking command:\n"
                msg += "%s\n" % mkdircommand + folder
                msg += "Exception: %s\n" % str(ex)
                msg += "Go on anyway..."
                print(msg)

    def createRemoveFileCommand(self, pfn):
        """
        handle both srm and file pfn types
        """
        if pfn.startswith("srm://"):
            return "srmrm -2 -retry_num=0 %s" % pfn
        elif pfn.startswith("file:"):
            return "/bin/rm -f %s" % pfn.replace("file://", "", 1)
        else:
            return StageOutImpl.createRemoveFileCommand(self, pfn)

    def createStageOutCommand(self, sourcePFN, targetPFN, options=None, checksums=None):
        """
        _createStageOutCommand_

        Build an srmcp command

        srmcp options used (so hard to find documentation for it...):
          -2                enables srm protocol version 2
          -report           path to the report file
          -retry_num        number of retries before before client gives up
          -request_lifetime request lifetime in seconds
        """
        result = "#!/bin/sh\n"
        result += "REPORT_FILE=`pwd`/srm.report.$$\n"
        result += "srmcp -2 -report=$REPORT_FILE -retry_num=0 -request_lifetime=2400"

        if options != None:
            result += " %s " % options
        result += " %s " % sourcePFN
        result += " %s" % targetPFN
        result += " 2>&1 | tee srm.output.$$ \n"

        if _CheckExitCodeOption:
            result += """
            EXIT_STATUS=`cat $REPORT_FILE | cut -f3 -d" "`
            echo "srmcp exit status: $EXIT_STATUS"
            if [[ "X$EXIT_STATUS" == "X" ]] && [[ `grep -c SRM_INVALID_PATH srm.output.$$` != 0 ]]; then
                echo "ERROR: srmcp failed with SRM_INVALID_PATH"
                exit 1   # dir does not exist
            elif [[ $EXIT_STATUS != 0 ]]; then
               echo "ERROR: srmcp exited with $EXIT_STATUS"
               echo "Cleaning up failed file:"
               %s
               exit $EXIT_STATUS
            fi

            """ % self.createRemoveFileCommand(targetPFN)

        if self.stageIn:
            remotePFN, localPFN = sourcePFN, targetPFN.replace("file://", "", 1)
        else:
            remotePFN, localPFN = targetPFN, sourcePFN.replace("file://", "", 1)
        remotePath = None
        SFN = '?SFN='
        sfn_idx = remotePFN.find(SFN)
        if sfn_idx >= 0:
            remotePath = remotePFN[sfn_idx + 5:]
        r = re.compile('srm://([A-Za-z\-\.0-9]*)(:[0-9]*)?(/.*)')
        m = r.match(remotePFN)
        if not m:
            raise StageOutError("Unable to determine path from PFN for " \
                                "target %s." % remotePFN)
        if remotePath == None:
            remotePath = m.groups()[2]
        remoteHost = m.groups()[0]

        result += "FILE_SIZE=`stat -c %s"
        result += " %s `\n" % localPFN
        result += "echo \"Local File Size is: $FILE_SIZE\"\n"

        metadataCheck = \
            """
            SRM_OUTPUT=`srmls -recursion_depth=0 -retry_num=1 %s 2>/dev/null`
            SRM_SIZE=`echo $SRM_OUTPUT | grep '%s' | grep -v '%s' | awk '{print $1;}'`
            echo "SRM Size is $SRM_SIZE"
            if [[ $SRM_SIZE == $FILE_SIZE ]]; then
               exit 0
            else
               echo $SRM_OUTPUT
               echo "ERROR: Size Mismatch between local and SE. Cleaning up failed file..."
               %s
               exit 60311
            fi
            """ % (remotePFN, remotePath, remoteHost, self.createRemoveFileCommand(targetPFN))
        result += metadataCheck

        return result
    
    def createDebuggingCommand(self, sourcePFN, targetPFN, options=None, checksums=None):
        """
        Debug a failed smrv2 copy command for stageOut, without re-running it,
        providing information on the environment and the certifications

        :sourcePFN: str, PFN of the source file
        :targetPFN: str, destination PFN
        :options: str, additional options for copy command
        :checksums: dict, collect checksums according to the algorithms saved as keys
        """
        # Build the copy command for debugging purposes
        copyCommand = ""
        copyCommand += "srmcp -2 -report=$REPORT_FILE -retry_num=0 -request_lifetime=2400"

        if options != None:
            copyCommand += " %s " % options
        copyCommand += " %s " % sourcePFN
        copyCommand += " %s" % targetPFN
        copyCommand += " 2>&1 | tee srm.output.$$ \n"

        if self.stageIn:
            remotePFN, localPFN = sourcePFN, targetPFN.replace("file://", "", 1)
        else:
            remotePFN, localPFN = targetPFN, sourcePFN.replace("file://", "", 1)
        remotePath = None
        SFN = '?SFN='
        sfn_idx = remotePFN.find(SFN)
        if sfn_idx >= 0:
            remotePath = remotePFN[sfn_idx + 5:]
        r = re.compile('srm://([A-Za-z\-\.0-9]*)(:[0-9]*)?(/.*)')
        m = r.match(remotePFN)
        if not m:
            raise StageOutError("Unable to determine path from PFN for " \
                                "target %s." % remotePFN)
        if remotePath == None:
            remotePath = m.groups()[2]

        result = self.debuggingTemplate.format(copy_command=copyCommand, source=localPFN, destination=remotePFN)
        return result

    def removeFile(self, pfnToRemove):
        """
        _removeFile_

        CleanUp pfn provided

        """
        command = "srmrm -2 -retry_num=0 %s" % pfnToRemove
        self.executeCommand(command)


registerStageOutImpl("srmv2", SRMV2Impl)



#!/usr/bin/env python
"""
Oracle implementation of GetRunLumiFile
"""




from WMComponent.DBS3Buffer.MySQL.DBSBufferFiles.GetRunLumiFile import GetRunLumiFile as MySQLGetRunLumiFile


class GetRunLumiFile(MySQLGetRunLumiFile):
    """
    Oracle implementation of GetRunLumiFile
    """



#!/usr/bin/env python
"""
_WMObject_t_

Testcase for the WMObject class

"""

import unittest
from WMCore.DataStructs.WMObject import WMObject
from unittest import TestCase

class WMObjectTest(unittest.TestCase):
    """
    _WMObjectTest_

    Testcase for the WMObject class

    """

    def setup(self):
        """
        Initial Setup for the Job Testcase

        P.S: For some weird reason, i couldnt instantiate a
        WMObject as an attribute of WMObject_t.
        So, I used it as a local variable on each method.
        Not the best programming technique I guess, but it works
        until i figure out what i am doing wrong.
        """
        #self.dummyWMObject = WMObject(self)
        pass
    def tearDown(self):
        """
        No tearDown method for this Testcase
        """
        pass

    def testMakeList(self):
        """
        Testcase for the makelist method of the WMObject Class
        """
        dummyWMObject = WMObject()
        #First case: Argument is already a list
        l = [1,2,3]
        assert dummyWMObject.makelist(l) == l, \
                'method makelist return value doesn\'t match ' \
                '- list argument test'
        #Second case: Argument is not a list
        a = 'test'
        assert dummyWMObject.makelist(a) == [a], \
                'method makelist return value doesn\'t match ' \
                '- common argument test'

    def testMakeSet(self):
        """
        Testcase for the makeset method of the WMObject Class
        """
        dummyWMObject = WMObject()
        #First case: Argument is already a Set
        s = set('1,2,3')
        assert dummyWMObject.makeset(s) == s, \
                'method makeset return value doesn\'t match ' \
                '- list argument test'
        #Second case: Argument is not a list
        b = [1,2,3]
        assert dummyWMObject.makeset(b) == set(b), \
            'method makeset return value doesn\'t match ' \
            '- common argument test'

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
_Subscription_t_

Unit tests for the WMBS Subscription class and all it's DAOs.
"""
from __future__ import division, print_function
import threading
import time
import unittest
from functools import reduce

from WMCore.DAOFactory import DAOFactory
from WMCore.DataStructs.Run import Run
from WMCore.WMBS.CreateWMBSBase import CreateWMBSBase
from WMCore.WMBS.File import File
from WMCore.WMBS.Fileset import Fileset
from WMCore.WMBS.Job import Job
from WMCore.WMBS.JobGroup import JobGroup
from WMCore.WMBS.Subscription import Subscription
from WMCore.WMBS.Workflow import Workflow
from WMQuality.TestInit import TestInit


class SubscriptionTest(unittest.TestCase):
    def setUp(self):
        """
        _setUp_

        Setup the database and logging connection.  Try to create all of the
        WMBS tables.  Also, create some dummy locations.
        """
        self.testInit = TestInit(__file__)
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection()
        self.testInit.setSchema(customModules=["WMCore.WMBS"],
                                useDefault=False)

        myThread = threading.currentThread()
        self.daofactory = DAOFactory(package="WMCore.WMBS",
                                     logger=myThread.logger,
                                     dbinterface=myThread.dbi)

        locationAction = self.daofactory(classname="Locations.New")
        locationAction.execute(siteName="site1", pnn="goodse.cern.ch")
        locationAction.execute(siteName="site2", pnn="testse.cern.ch")
        locationAction.execute(siteName="site3", pnn="badse.cern.ch")

        stateDAO = self.daofactory(classname="Jobs.GetStateID")
        self.stateID = stateDAO.execute('cleanout')

        return

    def tearDown(self):
        """
        _tearDown_

        Drop all the WMBS tables.
        """
        self.testInit.clearDatabase()

    def createSubscriptionWithFileABC(self):
        """
        _createSubscriptionWithFileABC_

        Create a subscription where the input fileset has three files.  Also
        create a second subscription that has acquired two of the files.
        """
        testWorkflow = Workflow(spec="spec.xml", owner="Simon",
                                name="wf001", task="Test")
        testWorkflow.create()
        testWorkflow2 = Workflow(spec="specBOGUS.xml", owner="Simon",
                                 name="wfBOGUS", task="Test")
        testWorkflow2.create()

        testFileA = File(lfn="/this/is/a/lfnA", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileA.addRun(Run(1, *[45]))

        testFileB = File(lfn="/this/is/a/lfnB", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileB.addRun(Run(1, *[45]))

        testFileC = File(lfn="/this/is/a/lfnC", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileC.addRun(Run(2, *[48]))

        testFileA.create()
        testFileB.create()
        testFileC.create()

        testFileset = Fileset(name="TestFileset")
        testFileset.create()

        testFileset.addFile(testFileA)
        testFileset.addFile(testFileB)
        testFileset.addFile(testFileC)
        testFileset.commit()

        testSubscription = Subscription(fileset=testFileset,
                                        workflow=testWorkflow)
        testSubscription2 = Subscription(fileset=testFileset,
                                         workflow=testWorkflow2)
        testSubscription2.create()
        testSubscription2.acquireFiles([testFileA, testFileB])

        return (testSubscription, testFileset, testWorkflow, testFileA,
                testFileB, testFileC)

    def createParentageScenario(self):
        """
        _createParentageScenario_

        Populate the DB with two workflows,
        with different filesets but one file in common.
        For one workflow this file belongs to the input fileset
        but for the other it is the parent of one of the files
        in its input dataset
        """

        # Create several files and set parentage relations
        # The genealogy goes like this
        # fileA and fileB belong to fileset1
        # fileC belongs to fileset2
        # fileB is a parent of fileC
        # fileD is parent of fileA and doesn't belong to any fileset
        testFileA = File(lfn="/this/is/a/lfnA", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileA.addRun(Run(1, *[45]))

        testFileB = File(lfn="/this/is/a/lfnB", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileB.addRun(Run(1, *[45]))

        testFileC = File(lfn="/this/is/a/lfnC", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileC.addRun(Run(2, *[48]))

        testFileD = File(lfn="/this/is/a/lfnD", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileD.addRun(Run(2, *[48]))

        testFileA.create()
        testFileB.create()
        testFileC.create()
        testFileD.create()

        testFileA.addParent(testFileD['lfn'])
        testFileC.addParent(testFileB['lfn'])

        testFileset1 = Fileset(name="TestFileset1")
        testFileset1.create()

        testFileset1.addFile(testFileA)
        testFileset1.addFile(testFileB)
        testFileset1.commit()

        testFileset2 = Fileset(name="TestFileset2")
        testFileset2.create()

        testFileset2.addFile(testFileC)
        testFileset2.commit()

        # Now to the workflows and subscriptions

        testWorkflow1 = Workflow(spec="specA.xml", owner="Simon",
                                 name="wf001", task="Test")
        testWorkflow1.create()
        testWorkflow2 = Workflow(spec="specB.xml", owner="Simon",
                                 name="wf002", task="Test")
        testWorkflow2.create()

        testSubscription1 = Subscription(fileset=testFileset1,
                                         workflow=testWorkflow1)
        testSubscription1.create()

        testSubscription2 = Subscription(fileset=testFileset2,
                                         workflow=testWorkflow2)
        testSubscription2.create()

        return {'Subscriptions': [testSubscription1, testSubscription2],
                'Workflows': [testWorkflow1, testWorkflow2],
                'Filesets': [testFileset1, testFileset2],
                'Files': [testFileA, testFileB, testFileC, testFileD]}

    def testCreateDeleteExists(self):
        """
        _testCreateDeleteExists_

        Create and delete a subscription and use the exists() method to
        determine if the create()/delete() methods were successful.
        """
        (testSubscription, testFileset, testWorkflow,
         testFileA, testFileB, testFileC) = self.createSubscriptionWithFileABC()

        self.assertFalse(testSubscription.exists(), "Subscription exists before it was created")

        testSubscription.create()

        self.assertTrue(testSubscription.exists() >= 0, "Subscription does not exist after it was created")

        testSubscription.delete()

        self.assertFalse(testSubscription.exists(), "Subscription exists after it was deleted")

        testFileset.delete()
        testFileA.delete()
        testFileB.delete()
        testFileC.delete()
        testWorkflow.delete()
        return

    def testCreateTransaction(self):
        """
        _testCreateTransaction_

        Create a subscription and commit it to the database.  Rollback the
        database connection and verify that the subscription is no longer
        there.
        """
        (testSubscription, testFileset, testWorkflow,
         testFileA, testFileB, testFileC) = self.createSubscriptionWithFileABC()

        self.assertFalse(testSubscription.exists(), "Subscription exists before it was created")

        myThread = threading.currentThread()
        myThread.transaction.begin()

        testSubscription.create()

        self.assertTrue(testSubscription.exists() >= 0, "Subscription does not exist after it was created")

        myThread.transaction.rollback()

        self.assertFalse(testSubscription.exists(), "Subscription exists after transaction was rolled back.")

        testFileset.delete()
        testFileA.delete()
        testFileB.delete()
        testFileC.delete()
        testWorkflow.delete()
        return

    def testDeleteTransaction(self):
        """
        _testDeleteTransaction_

        Create a subscription and commit it to the database.  Begin a new
        transactions and delete the subscription.  Verify that the subscription
        is no longer in the database and then roll back the transaction.  Verify
        that the subscription is once again in the database.
        """
        (testSubscription, testFileset, testWorkflow,
         testFileA, testFileB, testFileC) = self.createSubscriptionWithFileABC()

        self.assertFalse(testSubscription.exists(), "Subscription exists before it was created")

        testSubscription.create()

        self.assertTrue(testSubscription.exists() >= 0, "Subscription does not exist after it was created")

        myThread = threading.currentThread()
        myThread.transaction.begin()

        testSubscription.delete()

        self.assertFalse(testSubscription.exists(), "Subscription exists after it was deleted")

        myThread.transaction.rollback()

        self.assertTrue(testSubscription.exists() >= 0, "Subscription does not exist after roll back.")

        testFileset.delete()
        testFileA.delete()
        testFileB.delete()
        testFileC.delete()
        testWorkflow.delete()
        return

    def testFailFiles(self):
        """
        _testFailFiles_

        Create a subscription and fail a couple of files in it's fileset.  Test
        to make sure that only the failed files are marked as failed.
        """
        (testSubscription, testFileset, testWorkflow,
         testFileA, testFileB, testFileC) = self.createSubscriptionWithFileABC()

        testSubscription.create()

        dummyWorkflow = Workflow(spec="spec1.xml", owner="Simon",
                                 name="wf002", task='Test')
        dummyWorkflow.create()

        failSubscription = Subscription(fileset=testFileset,
                                        workflow=dummyWorkflow)
        failSubscription.create()
        failSubscription.failFiles(failSubscription.filesOfStatus("Available"))

        testSubscription.failFiles([testFileA, testFileC])
        failedFiles = testSubscription.filesOfStatus(status="Failed")

        goldenFiles = [testFileA, testFileC]
        for failedFile in failedFiles:
            assert failedFile in goldenFiles, \
                "Unknown failed files"
            goldenFiles.remove(failedFile)

        assert len(goldenFiles) == 0, \
            "Missing failed files"

        testSubscription.delete()
        testWorkflow.delete()
        testFileset.delete()
        testFileA.delete()
        testFileB.delete()
        testFileC.delete()
        return

    def testFailFilesTransaction(self):
        """
        _testFailFilesTransaction_

        Create a subscription and fail some files that are in it's input
        fileset.  Rollback the subscription and verify that the files are
        no longer failed.
        """
        (testSubscription, testFileset, testWorkflow,
         testFileA, testFileB, testFileC) = self.createSubscriptionWithFileABC()

        testSubscription.create()

        myThread = threading.currentThread()
        myThread.transaction.begin()

        testSubscription.failFiles([testFileA, testFileC])
        failedFiles = testSubscription.filesOfStatus(status="Failed")

        goldenFiles = [testFileA, testFileC]
        for failedFile in failedFiles:
            assert failedFile in goldenFiles, \
                "Unknown failed files"
            goldenFiles.remove(failedFile)

        assert len(goldenFiles) == 0, \
            "Missing failed files"

        myThread.transaction.rollback()

        failedFiles = testSubscription.filesOfStatus(status="Failed")

        assert len(failedFiles) == 0, \
            "Transaction did not roll back failed files"

        testSubscription.delete()
        testWorkflow.delete()
        testFileset.delete()
        testFileA.delete()
        testFileB.delete()
        testFileC.delete()
        return

    def testCompleteFiles(self):
        """
        _testCompleteFiles_

        Create a subscription and complete a couple of files in it's fileset.  Test
        to make sure that only the completed files are marked as complete.
        """
        (testSubscription, testFileset, testWorkflow,
         testFileA, testFileB, testFileC) = self.createSubscriptionWithFileABC()

        testSubscription.create()

        dummyWorkflow = Workflow(spec="spec2.xml", owner="Simon",
                                 name="wf003", task='Test')
        dummyWorkflow.create()

        completeSubscription = Subscription(fileset=testFileset,
                                            workflow=dummyWorkflow)
        completeSubscription.create()
        completeSubscription.completeFiles(completeSubscription.filesOfStatus("Available"))

        testSubscription.completeFiles([testFileA, testFileC])
        completedFiles = testSubscription.filesOfStatus(status="Completed")

        goldenFiles = [testFileA, testFileC]
        for completedFile in completedFiles:
            assert completedFile in goldenFiles, \
                "Unknown completed file"
            goldenFiles.remove(completedFile)

        assert len(goldenFiles) == 0, \
            "Missing completed files"

        testSubscription.delete()
        testWorkflow.delete()
        testFileset.delete()
        testFileA.delete()
        testFileB.delete()
        testFileC.delete()
        return

    def testCompleteFilesTransaction(self):
        """
        _testCompleteFilesTransaction_

        Create a subscription and complete some files that are in it's input
        fileset.  Rollback the transaction and verify that the files are no
        longer marked as complete.
        """
        (testSubscription, testFileset, testWorkflow,
         testFileA, testFileB, testFileC) = self.createSubscriptionWithFileABC()

        testSubscription.create()

        myThread = threading.currentThread()
        myThread.transaction.begin()

        testSubscription.completeFiles([testFileA, testFileC])
        completedFiles = testSubscription.filesOfStatus(status="Completed")

        goldenFiles = [testFileA, testFileC]
        for completedFile in completedFiles:
            assert completedFile in goldenFiles, \
                "Unknown completed file"
            goldenFiles.remove(completedFile)

        assert len(goldenFiles) == 0, \
            "Missing completed files"

        myThread.transaction.rollback()

        completedFiles = testSubscription.filesOfStatus(status="Completed")

        assert len(completedFiles) == 0, \
            "Transaction didn't roll back completed files."

        testSubscription.delete()
        testWorkflow.delete()
        testFileset.delete()
        testFileA.delete()
        testFileB.delete()
        testFileC.delete()
        return

    def testAcquireFiles(self):
        """
        _testAcquireFiles_

        Create a subscription and acquire a couple of files in it's fileset.  Test
        to make sure that only the acquired files are marked as acquired.
        """
        (testSubscription, testFileset, testWorkflow,
         testFileA, testFileB, testFileC) = self.createSubscriptionWithFileABC()

        testSubscription.create()

        dummyWorkflow = Workflow(spec="spec2.xml", owner="Simon",
                                 name="wf003", task='Test')
        dummyWorkflow.create()

        acquireSubscription = Subscription(fileset=testFileset,
                                           workflow=dummyWorkflow)
        acquireSubscription.create()
        acquireSubscription.acquireFiles()

        testSubscription.acquireFiles([testFileA, testFileC])
        acquiredFiles = testSubscription.filesOfStatus(status="Acquired")

        goldenFiles = [testFileA, testFileC]
        for acquiredFile in acquiredFiles:
            assert acquiredFile in goldenFiles, \
                "Unknown acquired file"
            goldenFiles.remove(acquiredFile)

        assert len(goldenFiles) == 0, \
            "Missing acquired files"

        testSubscription.delete()
        testWorkflow.delete()
        testFileset.delete()
        testFileA.delete()
        testFileB.delete()
        testFileC.delete()
        return

    def testAcquireFilesTransaction(self):
        """
        _testAcquireFilesTransaction_

        Create a subscription and acquire some files from it's input fileset.
        Rollback the transaction and verify that the files are no longer marked
        as acquired.
        """
        (testSubscription, testFileset, testWorkflow,
         testFileA, testFileB, testFileC) = self.createSubscriptionWithFileABC()

        testSubscription.create()

        myThread = threading.currentThread()
        myThread.transaction.begin()

        testSubscription.acquireFiles([testFileA, testFileC])
        acquiredFiles = testSubscription.filesOfStatus(status="Acquired")

        goldenFiles = [testFileA, testFileC]
        for acquiredFile in acquiredFiles:
            assert acquiredFile in goldenFiles, \
                "Unknown acquired file"
            goldenFiles.remove(acquiredFile)

        assert len(goldenFiles) == 0, \
            "Missing acquired files"

        myThread.transaction.rollback()
        acquiredFiles = testSubscription.filesOfStatus(status="Acquired")

        assert len(acquiredFiles) == 0, \
            "Transaction didn't roll back acquired files."

        testSubscription.delete()
        testWorkflow.delete()
        testFileset.delete()
        testFileA.delete()
        testFileB.delete()
        testFileC.delete()
        return

    def testAvailableFiles(self):
        """
        _testAvailableFiles_

        Create a subscription and mark a couple files as failed, complete and
        acquired.  Test to make sure that the remainder of the files show up
        as available.
        """
        testWorkflow = Workflow(spec="spec.xml", owner="Simon",
                                name="wf001", task='Test')
        testWorkflow.create()

        testFileA = File(lfn="/this/is/a/lfnA", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileB = File(lfn="/this/is/a/lfnB", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileC = File(lfn="/this/is/a/lfnC", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileD = File(lfn="/this/is/a/lfnD", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileE = File(lfn="/this/is/a/lfnE", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileF = File(lfn="/this/is/a/lfnF", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileA.create()
        testFileB.create()
        testFileC.create()
        testFileD.create()
        testFileE.create()
        testFileF.create()

        testFileset = Fileset(name="TestFileset")
        testFileset.create()

        testFileset.addFile(testFileA)
        testFileset.addFile(testFileB)
        testFileset.addFile(testFileC)
        testFileset.addFile(testFileD)
        testFileset.addFile(testFileE)
        testFileset.addFile(testFileF)
        testFileset.commit()

        testSubscription = Subscription(fileset=testFileset,
                                        workflow=testWorkflow)
        testSubscription.create()

        testSubscription.acquireFiles([testFileA])
        testSubscription.completeFiles([testFileB])
        testSubscription.failFiles([testFileC])
        availableFiles = testSubscription.availableFiles()

        goldenFiles = [testFileD, testFileE, testFileF]
        for availableFile in availableFiles:
            assert availableFile in goldenFiles, \
                "Unknown available file"
            assert len(availableFile["locations"]) == 1, \
                "Wrong number of available files."
            assert "goodse.cern.ch" in availableFile["locations"], \
                "Wrong SE name in file location."
            goldenFiles.remove(availableFile)

        assert len(goldenFiles) == 0, \
            "Missing available files"

        testSubscription.delete()
        testWorkflow.delete()
        testFileset.delete()
        testFileA.delete()
        testFileB.delete()
        testFileC.delete()
        testFileD.delete()
        testFileE.delete()
        testFileF.delete()
        return

    def testAvailableFilesMeta(self):
        """
        _testAvailableFilesMeta_

        Create a subscription and mark a couple files as failed, complete and
        acquired.  Test to make sure that the remainder of the files show up
        as available using the GetAvailableFilesMeta DAO object.
        """
        testWorkflow = Workflow(spec="spec.xml", owner="Simon",
                                name="wf001", task='Test')
        testWorkflow.create()

        testFileA = File(lfn="/this/is/a/lfnA", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileB = File(lfn="/this/is/a/lfnB", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileC = File(lfn="/this/is/a/lfnC", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileD = File(lfn="/this/is/a/lfnD", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileD.addRun(Run(1, *[45]))
        testFileD.addRun(Run(2, *[45]))
        testFileE = File(lfn="/this/is/a/lfnE", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileE.addRun(Run(1, *[45]))
        testFileE.addRun(Run(2, *[45]))
        testFileF = File(lfn="/this/is/a/lfnF", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileF.addRun(Run(1, *[45]))
        testFileF.addRun(Run(2, *[45]))
        testFileA.create()
        testFileB.create()
        testFileC.create()
        testFileD.create()
        testFileE.create()
        testFileF.create()

        testFileset = Fileset(name="TestFileset")
        testFileset.create()

        testFileset.addFile(testFileA)
        testFileset.addFile(testFileB)
        testFileset.addFile(testFileC)
        testFileset.addFile(testFileD)
        testFileset.addFile(testFileE)
        testFileset.addFile(testFileF)
        testFileset.commit()

        testSubscription = Subscription(fileset=testFileset,
                                        workflow=testWorkflow)
        testSubscription.create()

        testSubscription.acquireFiles([testFileA])
        testSubscription.completeFiles([testFileB])
        testSubscription.failFiles([testFileC])

        availAction = self.daofactory(classname="Subscriptions.GetAvailableFilesMeta")
        availableFiles = availAction.execute(subscription=testSubscription["id"])

        testFileDDict = {"id": testFileD["id"], "lfn": testFileD["lfn"],
                         "size": testFileD["size"], "events": testFileD["events"],
                         "run": 1}
        testFileEDict = {"id": testFileE["id"], "lfn": testFileE["lfn"],
                         "size": testFileE["size"], "events": testFileE["events"],
                         "run": 1}
        testFileFDict = {"id": testFileF["id"], "lfn": testFileF["lfn"],
                         "size": testFileF["size"], "events": testFileF["events"],
                         "run": 1}

        goldenFiles = [testFileDDict, testFileEDict, testFileFDict]
        for availableFile in availableFiles:
            assert availableFile in goldenFiles, \
                "Unknown available file: %s" % availableFile
            goldenFiles.remove(availableFile)

        assert len(goldenFiles) == 0, \
            "Missing available files"

        testSubscription.delete()
        testWorkflow.delete()
        testFileset.delete()
        testFileA.delete()
        testFileB.delete()
        testFileC.delete()
        testFileD.delete()
        testFileE.delete()
        testFileF.delete()
        return

    def testAvailableFilesTransaction(self):
        """
        _testAvailableFilesTransaction_

        Create a subscription and mark a couple of it's input files as
        complete, failed and acquired.  Rollback the transactions and verify
        that all the files are listed as available.
        """
        testWorkflow = Workflow(spec="spec.xml", owner="Simon",
                                name="wf001", task='Test')
        testWorkflow.create()

        testFileA = File(lfn="/this/is/a/lfnA", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileB = File(lfn="/this/is/a/lfnB", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileC = File(lfn="/this/is/a/lfnC", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileD = File(lfn="/this/is/a/lfnD", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileE = File(lfn="/this/is/a/lfnE", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileF = File(lfn="/this/is/a/lfnF", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileA.create()
        testFileB.create()
        testFileC.create()
        testFileD.create()
        testFileE.create()
        testFileF.create()

        testFileset = Fileset(name="TestFileset")
        testFileset.create()

        testFileset.addFile(testFileA)
        testFileset.addFile(testFileB)
        testFileset.addFile(testFileC)
        testFileset.addFile(testFileD)
        testFileset.addFile(testFileE)
        testFileset.addFile(testFileF)
        testFileset.commit()

        testSubscription = Subscription(fileset=testFileset,
                                        workflow=testWorkflow)
        testSubscription.create()

        myThread = threading.currentThread()
        myThread.transaction.begin()

        testSubscription.acquireFiles([testFileA])
        testSubscription.completeFiles([testFileB])
        testSubscription.failFiles([testFileC])
        availableFiles = testSubscription.availableFiles()

        goldenFiles = [testFileD, testFileE, testFileF]
        for availableFile in availableFiles:
            assert availableFile in goldenFiles, \
                "Unknown available file"
            goldenFiles.remove(availableFile)

        assert len(goldenFiles) == 0, \
            "Missing available files"

        myThread.transaction.rollback()
        availableFiles = testSubscription.availableFiles()

        goldenFiles = [testFileA, testFileB, testFileC, testFileD, testFileE,
                       testFileF]

        for availableFile in availableFiles:
            assert availableFile in goldenFiles, \
                "Unknown available file"
            goldenFiles.remove(availableFile)

        assert len(goldenFiles) == 0, \
            "Missing available files after rollback."

        testSubscription.delete()
        testWorkflow.delete()
        testFileset.delete()
        testFileA.delete()
        testFileB.delete()
        testFileC.delete()
        testFileD.delete()
        testFileE.delete()
        testFileF.delete()
        return

    def testLoad(self):
        """
        _testLoad_

        Create a subscription and save it to the database.  Test the various
        load methods to make sure that everything saves/loads.
        """
        (testSubscriptionA, testFileset, testWorkflow,
         dummyTestFileA, dummyTestFileB, dummyTestFileC) = self.createSubscriptionWithFileABC()

        testSubscriptionA.create()

        testSubscriptionB = Subscription(id=testSubscriptionA["id"])
        testSubscriptionC = Subscription(workflow=testSubscriptionA["workflow"],
                                         fileset=testSubscriptionA["fileset"],
                                         type=testSubscriptionA["type"])
        testSubscriptionB.load()
        testSubscriptionC.load()

        self.assertTrue(isinstance(testSubscriptionB["id"], int),
                        "Subscription id is not an int.")
        self.assertTrue(isinstance(testSubscriptionB["workflow"].id, int),
                        "Subscription workflow id is not an int.")
        self.assertTrue(isinstance(testSubscriptionC["workflow"].id, int),
                        "Subscription workflow id is not an int.")
        self.assertTrue(isinstance(testSubscriptionB["fileset"].id, int),
                        "Subscription fileset id is not an int.")
        self.assertTrue(isinstance(testSubscriptionC["fileset"].id, int),
                        "Subscription fileset id is not an int.")

        self.assertEqual(testWorkflow.id, testSubscriptionB["workflow"].id,
                         "Subscription load by ID didn't load workflow correctly")
        self.assertEqual(testFileset.id, testSubscriptionB["fileset"].id,
                         "Subscription load by ID didn't load fileset correctly")
        self.assertEqual(testSubscriptionA["id"], testSubscriptionC["id"],
                         "Subscription didn't load ID correctly.")

        return

    def testLoadData(self):
        """
        _testLoadData_

        Test the Subscription's loadData() method to make sure that everything
        that should be loaded is loaded correctly.
        """
        (testSubscriptionA, testFileset, testWorkflow,
         testFileA, testFileB, testFileC) = self.createSubscriptionWithFileABC()

        testSubscriptionA.create()

        testSubscriptionB = Subscription(id=testSubscriptionA["id"])
        testSubscriptionC = Subscription(workflow=testSubscriptionA["workflow"],
                                         fileset=testSubscriptionA["fileset"])
        testSubscriptionB.loadData()
        testSubscriptionC.loadData()

        assert (testWorkflow.id == testSubscriptionB["workflow"].id) and \
               (testWorkflow.name == testSubscriptionB["workflow"].name) and \
               (testWorkflow.spec == testSubscriptionB["workflow"].spec) and \
               (testWorkflow.owner == testSubscriptionB["workflow"].owner), \
            "Workflow.LoadFromID Failed"

        assert testFileset.id == testSubscriptionB["fileset"].id, \
            "Load didn't load fileset id"

        assert testFileset.name == testSubscriptionB["fileset"].name, \
            "Load didn't load fileset name"

        goldenFiles = [testFileA, testFileB, testFileC]
        for filesetFile in testSubscriptionB["fileset"].files:
            assert filesetFile in goldenFiles, \
                "Unknown file in fileset"
            goldenFiles.remove(filesetFile)

        assert len(goldenFiles) == 0, \
            "Fileset is missing files"

        assert testSubscriptionA["id"] == testSubscriptionC["id"], \
            "Subscription didn't load ID correctly."

        return

    def testSubscriptionList(self):
        """
        _testSubscriptionList_

        Create two subscriptions and verify that the Subscriptions.List DAO
        object returns their IDs.
        """
        testWorkflowA = Workflow(spec="spec.xml", owner="Simon",
                                 name="wf001", task='Test')
        testWorkflowB = Workflow(spec="spec2.xml", owner="Simon",
                                 name="wf002", task='Test')
        testWorkflowA.create()
        testWorkflowB.create()

        testFileset = Fileset(name="TestFileset")
        testFileset.create()

        testSubscriptionA = Subscription(fileset=testFileset,
                                         workflow=testWorkflowA)
        testSubscriptionB = Subscription(fileset=testFileset,
                                         workflow=testWorkflowB)
        testSubscriptionA.create()
        testSubscriptionB.create()

        myThread = threading.currentThread()
        daofactory = DAOFactory(package="WMCore.WMBS",
                                logger=myThread.logger,
                                dbinterface=myThread.dbi)

        subListAction = daofactory(classname="Subscriptions.List")
        subIDs = subListAction.execute()

        assert len(subIDs) == 2, \
            "Too many subscriptions returned."

        assert testSubscriptionA["id"] in subIDs, \
            "Subscription A is missing."

        assert testSubscriptionB["id"] in subIDs, \
            "Subscription B is missing."

        return

    def testGetNumberOfJobsPerSite(self):
        """
        Test for a JobCreator specific function

        """

        (testSubscription, dummyTestFileset, dummyTestWorkflow,
         testFileA, testFileB, testFileC) = self.createSubscriptionWithFileABC()

        testSubscription.create()

        testJobGroup = JobGroup(subscription=testSubscription)
        testJobGroup.create()

        jobA = Job(name='testA')
        jobA.addFile(testFileA)
        jobA["location"] = "site1"
        jobA.create(testJobGroup)

        jobB = Job(name='testB')
        jobB.addFile(testFileB)
        jobB["location"] = "site1"
        jobB.create(testJobGroup)

        jobC = Job(name='testC')
        jobC.addFile(testFileC)
        jobC["location"] = "site1"
        jobC.create(testJobGroup)

        testJobGroup.add(jobA)
        testJobGroup.add(jobB)
        testJobGroup.add(jobC)

        testJobGroup.commit()

        nJobs = testSubscription.getNumberOfJobsPerSite('site1', 'new')

        self.assertEqual(nJobs, 3)

        nZero = testSubscription.getNumberOfJobsPerSite('site3', 'new')

        self.assertEqual(nZero, 0)

        return

    def testListIncompleteDAO(self):
        """
        _testListIncompeteDAO_

        Test the Subscription.ListIncomplete DAO object that returns a list of
        the subscriptions that have not completed processing all files.
        """
        (testSubscription, dummyTestFileset, dummyTestWorkflow,
         testFileA, testFileB, testFileC) = self.createSubscriptionWithFileABC()
        testSubscription.create()

        subIncomplete = self.daofactory(classname="Subscriptions.ListIncomplete")

        incompleteSubs = subIncomplete.execute()

        self.assertEqual(len(incompleteSubs), 2,
                         "Wrong number of incomplete subscriptions returned: %s" % len(incompleteSubs))
        self.assertTrue(testSubscription["id"] in incompleteSubs, "Original subscription is missing.")

        otherSub = None
        for subId in incompleteSubs:
            if subId != testSubscription["id"]:
                otherSub = subId

        testSubscription.completeFiles([testFileA, testFileB, testFileC])

        incompleteSubs = subIncomplete.execute()

        self.assertEqual(len(incompleteSubs), 1, "Wrong number of incomplete subscriptions returned.")
        self.assertTrue(otherSub in incompleteSubs, "Wrong subscription ID returned.")

        return

    def testGetJobGroups(self):
        """
        _testGetJobGroups_

        Verify that the getJobGroups() method will return a list of JobGroups
        that contain jobs that have not been acquired/completed/failed.
        """
        (testSubscription, dummyTestFileset, dummyTestWorkflow,
         testFileA, testFileB, testFileC) = self.createSubscriptionWithFileABC()
        testSubscription.create()

        changeJobState = self.daofactory(classname="Jobs.ChangeState")

        testJobGroupA = JobGroup(subscription=testSubscription)
        testJobGroupA.create()

        testJobA = Job(name="TestJobA")
        testJobA.addFile(testFileA)

        testJobB = Job(name="TestJobB")
        testJobB.addFile(testFileB)

        testJobGroupA.add(testJobA)
        testJobGroupA.add(testJobB)
        testJobGroupA.commit()

        testJobGroupB = JobGroup(subscription=testSubscription)
        testJobGroupB.create()

        testFileD = File(lfn="/this/is/a/lfnD", size=1024, events=10)
        testFileD.addRun(Run(10, *[12312]))
        testFileD.create()

        testJobC = Job(name="TestJobC")
        testJobC.addFile(testFileC)

        testJobD = Job(name="TestJobD")
        testJobD.addFile(testFileD)

        testJobGroupB.add(testJobC)
        testJobGroupB.add(testJobD)
        testJobGroupB.commit()

        firstResult = testSubscription.getJobGroups()
        for jobGroup in [testJobGroupA.id, testJobGroupB.id]:
            assert jobGroup in firstResult, \
                "jobgroup %s is missing. " % jobGroup
            firstResult.remove(jobGroup)

        assert len(firstResult) == 0, \
            "Too monay job groups in result."

        testJobA["state"] = "created"
        changeJobState.execute([testJobA])

        secondResult = testSubscription.getJobGroups()
        for jobGroup in [testJobGroupA.id, testJobGroupB.id]:
            assert jobGroup in secondResult, \
                "jobgroup %s is missing. " % jobGroup
            secondResult.remove(jobGroup)

        assert len(secondResult) == 0, \
            "Too monay job groups in result."

        testJobB["state"] = "created"
        changeJobState.execute([testJobB])

        thirdResult = testSubscription.getJobGroups()
        for jobGroup in [testJobGroupB.id]:
            assert jobGroup in thirdResult, \
                "jobgroup %s is missing. " % jobGroup
            thirdResult.remove(jobGroup)

        assert len(thirdResult) == 0, \
            "Too monay job groups in result."

        return

    def testDeleteEverything(self):
        """
        _testDeleteEverything_

        Tests the delete function that should delete all component of a subscription
        """
        myThread = threading.currentThread()

        testWorkflow = Workflow(spec="spec.xml", owner="Simon",
                                name="wf001", task="Test")
        testWorkflow.create()

        testFileA = File(lfn="/this/is/a/lfnA", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileA.addRun(Run(1, *[45]))

        testFileB = File(lfn="/this/is/a/lfnB", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileB.addRun(Run(1, *[45]))

        testFileC = File(lfn="/this/is/a/lfnC", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileC.addRun(Run(2, *[48]))

        testFileD = File(lfn="/this/is/a/lfnD", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileD.addRun(Run(2, *[48]))

        testFile1 = File(lfn="/this/is/a/lfn1", size=1024, events=20,
                         locations={"goodse.cern.ch"}, merged=False)
        testFile1.addRun(Run(2, *[48]))

        testFile2 = File(lfn="/this/is/a/lfn2", size=1024, events=20,
                         locations={"goodse.cern.ch"}, merged=False)
        testFile2.addRun(Run(2, *[48]))

        testFileA.create()
        testFileB.create()
        testFileC.create()
        testFileD.create()
        testFile1.create()
        testFileA.addChild(testFile1['lfn'])

        testFileset = Fileset(name="TestFileset")
        testFileset.create()

        testFileset2 = Fileset(name="TestFileset2")
        testFileset2.create()

        testFileset.addFile(testFileA)
        testFileset.addFile(testFileB)
        testFileset.addFile(testFileC)
        testFileset.addFile(testFileD)
        testFileset.commit()

        testFilesetQ = Fileset(name="TestFilesetQ")
        testFilesetQ.create()
        testMergedFilesetQ = Fileset(name="TestMergedFilesetQ")
        testMergedFilesetQ.create()

        testWorkflow.addOutput(outputIdentifier='a',
                               outputFileset=testFilesetQ,
                               mergedOutputFileset=testMergedFilesetQ)

        testFileset2.addFile(testFileD)
        testFileset2.commit()

        testSubscription = Subscription(fileset=testFileset,
                                        workflow=testWorkflow)
        testSubscription.create()
        testSubscription2 = Subscription(fileset=testFileset2,
                                         workflow=testWorkflow)
        testSubscription2.create()

        testJobGroupA = JobGroup(subscription=testSubscription)
        testJobGroupA.create()

        testJobA = Job(name="TestJobA")
        testJobA.addFile(testFileA)

        testJobB = Job(name="TestJobB")
        testJobB.addFile(testFileB)

        testJobGroupA.add(testJobA)
        testJobGroupA.add(testJobB)
        testJobGroupA.output.addFile(testFile1)
        testJobGroupA.output.addFile(testFile2)
        testJobGroupA.output.commit()

        testJobGroupA.commit()
        testSubscription.save()

        testSubscription.deleteEverything()

        self.assertEqual(testSubscription.exists(), False)
        self.assertEqual(testWorkflow.exists(), 1)
        self.assertEqual(testFileset.exists(), False)
        self.assertEqual(testFileset2.exists(), 2)

        result = myThread.dbi.processData("SELECT * FROM wmbs_job")[0].fetchall()
        self.assertEqual(len(result), 0)
        result = myThread.dbi.processData("SELECT * FROM wmbs_jobgroup")[0].fetchall()
        self.assertEqual(len(result), 0)
        self.assertFalse(testJobGroupA.output.exists())
        self.assertEqual(testFile1.exists(), False)
        self.assertEqual(testFile2.exists(), False)
        self.assertFalse(testFilesetQ.exists())
        self.assertEqual(testFileA.exists(), False)
        self.assertEqual(testFileB.exists(), False)
        self.assertEqual(testFileC.exists(), False)
        self.assertEqual(testFileD.exists(), 4)

    def testIsFileCompleted(self):
        """
        _testIsFileCompleted_

        Test file completion markings
        """
        (testSubscription, testFileset, testWorkflow,
         testFileA, testFileB, testFileC) = self.createSubscriptionWithFileABC()

        testSubscription.create()

        myThread = threading.currentThread()
        myThread.transaction.begin()

        testSubscription.completeFiles([testFileA, testFileC])

        self.assertTrue(testSubscription.isFileCompleted(testFileA), "file A should be completed")
        self.assertTrue(testSubscription.isFileCompleted([testFileA, testFileC]), "file A, C should be completed")
        self.assertFalse(testSubscription.isFileCompleted([testFileA, testFileB, testFileC]),
                         "file A, B, C shouldn't be completed")

        self.assertFalse(testSubscription.isFileCompleted(testFileB), "file B shouldn't be completed")

        testSubscription.delete()
        testWorkflow.delete()
        testFileset.delete()
        testFileA.delete()
        testFileB.delete()
        testFileC.delete()
        return

    def testGetSubTypes(self):
        """
        _testGetSubTypes_

        Test the getSubTypes function
        """
        createBase = CreateWMBSBase()
        subTypes = createBase.subTypes

        getSubTypes = self.daofactory(classname="Subscriptions.GetSubTypes")
        result = getSubTypes.execute()

        self.assertEqual(result.sort(), subTypes.sort())

        return

    def testBulkCommit(self):
        """
        _testBulkCommit_

        Test committing everything in bulk
        """
        testWorkflow = Workflow(spec="spec.xml", owner="Simon",
                                name="wf001", task="Test")
        testWorkflow.create()

        testFileA = File(lfn="/this/is/a/lfnA", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileA.addRun(Run(1, *[45]))

        testFileB = File(lfn="/this/is/a/lfnB", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileB.addRun(Run(1, *[45]))

        testFileC = File(lfn="/this/is/a/lfnC", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileC.addRun(Run(2, *[48]))

        testFileD = File(lfn="/this/is/a/lfnD", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileD.addRun(Run(2, *[48]))

        testFileA.create()
        testFileB.create()
        testFileC.create()
        testFileD.create()

        testFileset = Fileset(name="TestFileset")
        testFileset.create()

        testFileset2 = Fileset(name="TestFileset2")
        testFileset2.create()

        testFileset.addFile(testFileA)
        testFileset.addFile(testFileB)
        testFileset.addFile(testFileC)
        testFileset.addFile(testFileD)
        testFileset.commit()

        testSubscription = Subscription(fileset=testFileset,
                                        workflow=testWorkflow)
        testSubscription.create()

        # Everything above here has to exist before we get started

        testJobGroupA = JobGroup(subscription=testSubscription)
        testJobGroupB = JobGroup(subscription=testSubscription)

        testJobA = Job(name="TestJobA")
        testJobA.addFile(testFileA)
        testJobA['mask']['FirstEvent'] = 101
        testJobA['mask']['LastEvent'] = 101
        testJobA['mask']['FirstLumi'] = 102
        testJobA['mask']['LastLumi'] = 102

        testJobB = Job(name="TestJobB")
        testJobB.addFile(testFileB)

        testJobC = Job(name="TestJobC")
        testJobC.addFile(testFileC)

        testJobD = Job(name="TestJobD")
        testJobD.addFile(testFileD)

        testJobGroupA.add(testJobA)
        testJobGroupA.add(testJobB)
        testJobGroupB.add(testJobC)
        testJobGroupB.add(testJobD)

        testSubscription.bulkCommit(jobGroups=[testJobGroupA, testJobGroupB])

        self.assertEqual(testJobA.exists() > 0, True)
        self.assertEqual(testJobB.exists() > 0, True)
        self.assertEqual(testJobC.exists() > 0, True)
        self.assertEqual(testJobD.exists() > 0, True)
        self.assertEqual(testJobGroupA.exists() > 0, True)
        self.assertEqual(testJobGroupB.exists() > 0, True)

        result = testSubscription.filesOfStatus(status="Acquired")
        self.assertEqual(len(result), 4,
                         'Should have acquired 4 files, instead have %i' % (len(result)))
        self.assertEqual(len(testJobGroupA.jobs), 2)

        testJob1 = Job(id=testJobA.exists())
        testJob1.loadData()
        self.assertEqual(testJob1['mask']['FirstEvent'], 101)
        self.assertEqual(testJob1['mask']['LastEvent'], 101)
        self.assertEqual(testJob1['mask']['FirstLumi'], 102)
        self.assertEqual(testJob1['mask']['LastLumi'], 102)

        # # Test that work units were created
        #
        # testRunLumi = Run(1, 45)
        # loadByFRL = WorkUnit(taskID=testWorkflow.id, fileid=testFileA['id'], runLumi=testRunLumi)
        # loadByFRL.load()
        #
        # self.assertGreater(loadByFRL['id'], 0)
        # self.assertEqual(loadByFRL['taskid'], testWorkflow.id)
        # self.assertEqual(loadByFRL['fileid'], testFileA['id'])
        # self.assertEqual(loadByFRL['last_unit_count'], 1)  # One lumi in this job
        #
        # testRunLumi = Run(2, 48)
        # loadByFRL = WorkUnit(taskID=testWorkflow.id, fileid=testFileD['id'], runLumi=testRunLumi)
        # loadByFRL.load()
        #
        # self.assertGreater(loadByFRL['id'], 0)
        # self.assertEqual(loadByFRL['taskid'], testWorkflow.id)
        # self.assertEqual(loadByFRL['fileid'], testFileD['id'])
        # self.assertEqual(loadByFRL['last_unit_count'], 1)  # One lumi in this job

        return

    # def testBulkCommit2(self):
    #     """
    #     _testBulkCommit2_
    #
    #     Test committing everything in bulk with a different set of parameters,
    #     making sure work units are done correctly
    #     """
    #
    #     testWorkflow = Workflow(spec="spec.xml", owner="Simon", name="wf001", task="Test")
    #     testWorkflow.create()
    #
    #     testFileA = File(lfn="/this/is/a/lfnA", size=1024, events=20, locations={"goodse.cern.ch"})
    #     testFileA.addRun(Run(1, *[45]))
    #
    #     testFileB = File(lfn="/this/is/a/lfnB", size=1024, events=20, locations={"goodse.cern.ch"})
    #     testFileB.addRun(Run(1, *[45]))
    #
    #     testFileC = File(lfn="/this/is/a/lfnC", size=1024, events=20, locations={"goodse.cern.ch"})
    #     testFileC.addRun(Run(2, *[46, 48, 50]))
    #
    #     testFileD = File(lfn="/this/is/a/lfnD", size=1024, events=20, locations={"goodse.cern.ch"})
    #     testFileD.addRun(Run(2, *[49, 51, 52]))
    #
    #     testFileA.create()
    #     testFileB.create()
    #     testFileC.create()
    #     testFileD.create()
    #
    #     testFileset = Fileset(name="TestFileset")
    #     testFileset.create()
    #
    #     testFileset.addFile(testFileA)
    #     testFileset.addFile(testFileB)
    #     testFileset.addFile(testFileC)
    #     testFileset.addFile(testFileD)
    #     testFileset.commit()
    #
    #     testSubscription = Subscription(fileset=testFileset, workflow=testWorkflow)
    #     testSubscription.create()
    #
    #     # Everything above here has to exist before we get started
    #
    #     testJobGroupA = JobGroup(subscription=testSubscription)
    #     testJobGroupB = JobGroup(subscription=testSubscription)
    #
    #     testJobA = Job(name="TestJobA")
    #     testJobA.addFile(testFileA)
    #
    #     testJobB = Job(name="TestJobB")
    #     testJobB.addFile(testFileB)
    #
    #     testJobC = Job(name="TestJobC")
    #     testJobC.addFile(testFileC)
    #
    #     testJobD = Job(name="TestJobD")
    #     testJobD.addFile(testFileD)
    #
    #     testJobGroupA.add(testJobA)
    #     testJobGroupA.add(testJobB)
    #     testJobGroupB.add(testJobC)
    #     testJobGroupB.add(testJobD)
    #
    #     testSubscription.bulkCommit(jobGroups=[testJobGroupA, testJobGroupB])
    #
    #     self.assertGreater(testJobA.exists(), 0)
    #     self.assertGreater(testJobB.exists(), 0)
    #     self.assertGreater(testJobC.exists(), 0)
    #     self.assertGreater(testJobD.exists(), 0)
    #     self.assertGreater(testJobGroupA.exists(), 0)
    #     self.assertGreater(testJobGroupB.exists(), 0)
    #
    #     result = testSubscription.filesOfStatus(status="Acquired")
    #     self.assertEqual(len(result), 4, 'Should have acquired 4 files, instead have %i' % len(result))
    #     self.assertEqual(len(testJobGroupA.jobs), 2)
    #
    #     testJob1 = Job(id=testJobA.exists())
    #     testJob1.loadData()
    #
    #     # Test that work units were created
    #
    #     testRunLumi = Run(1, 45)
    #     loadByFRL = WorkUnit(taskID=testWorkflow.id, fileid=testFileA['id'], runLumi=testRunLumi)
    #     loadByFRL.load()
    #
    #     self.assertGreater(loadByFRL['id'], 0)
    #     self.assertEqual(loadByFRL['taskid'], testWorkflow.id)
    #     self.assertEqual(loadByFRL['fileid'], testFileA['id'])
    #     self.assertEqual(loadByFRL['last_unit_count'], 1)  # One lumi in this job
    #
    #     testRunLumi = Run(2, 48)
    #     loadByFRL = WorkUnit(taskID=testWorkflow.id, fileid=testFileC['id'], runLumi=testRunLumi)
    #     loadByFRL.load()
    #
    #     self.assertGreater(loadByFRL['id'], 0)
    #     self.assertEqual(loadByFRL['taskid'], testWorkflow.id)
    #     self.assertEqual(loadByFRL['fileid'], testFileC['id'])
    #     self.assertEqual(loadByFRL['last_unit_count'], 3)  # Three lumis in this job
    #
    #     return

    def testFilesOfStatusByLimit(self):
        """
        _testFilesOfstatusByLimit_

        Create a subscription and mark a couple files as failed, complete and
        acquired.  Test to make sure that the remainder of the files show up
        as available.
        """
        testWorkflow = Workflow(spec="spec.xml", owner="Simon",
                                name="wf001", task='Test')
        testWorkflow.create()

        testFileA = File(lfn="/this/is/a/lfnA", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileB = File(lfn="/this/is/a/lfnB", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileC = File(lfn="/this/is/a/lfnC", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileD = File(lfn="/this/is/a/lfnD", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileE = File(lfn="/this/is/a/lfnE", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileF = File(lfn="/this/is/a/lfnF", size=1024, events=20,
                         locations={"goodse.cern.ch"})
        testFileA.create()
        testFileB.create()
        testFileC.create()
        testFileD.create()
        testFileE.create()
        testFileF.create()

        testFileset = Fileset(name="TestFileset")
        testFileset.create()

        testFileset.addFile(testFileA)
        testFileset.addFile(testFileB)
        testFileset.addFile(testFileC)
        testFileset.addFile(testFileD)
        testFileset.addFile(testFileE)
        testFileset.addFile(testFileF)
        testFileset.commit()

        testSubscription = Subscription(fileset=testFileset,
                                        workflow=testWorkflow)
        testSubscription.create()

        availableFiles = testSubscription.filesOfStatus("Available")
        self.assertEqual(len(availableFiles), 6)

        testSubscription.acquireFiles([testFileA, testFileB, testFileC, testFileD])
        availableFiles = testSubscription.filesOfStatus("Available")
        self.assertEqual(len(availableFiles), 2)

        files = testSubscription.filesOfStatus("Acquired")
        self.assertEqual(len(files), 4)

        testSubscription.completeFiles([testFileB, testFileC])

        files = testSubscription.filesOfStatus("Completed")

        testSubscription.failFiles([testFileA, testFileE])

        files = testSubscription.filesOfStatus("Failed")
        self.assertEqual(len(files), 2)

        testSubscription.delete()
        testWorkflow.delete()
        testFileset.delete()
        testFileA.delete()
        testFileB.delete()
        testFileC.delete()
        testFileD.delete()
        testFileE.delete()
        testFileF.delete()
        return

    def testJobs(self):
        """
        _testJobs_

        Test the Subscription.AllJob DAO turns the correct number of job
        """
        testFileset = Fileset(name="TestFileset")
        testFileset.create()

        testWorkflow = Workflow(spec="spec.xml", owner="Hassen",
                                name="wf001", task="Test")
        testWorkflow.create()
        testSubscription = Subscription(fileset=testFileset,
                                        workflow=testWorkflow)
        testSubscription.create()

        testJobGroupA = JobGroup(subscription=testSubscription)
        testJobGroupA.create()

        testJobA = Job(name="TestJobA")
        testJobB = Job(name="TestJobB")

        testJobGroupA.add(testJobA)
        testJobGroupA.add(testJobB)
        testJobGroupA.commit()

        getAllJobs = self.daofactory(classname="Subscriptions.Jobs")
        result = getAllJobs.execute(subscription=testSubscription["id"])

        assert len(result) == 2, \
            "Wrong number of jobs."

        testJobC = Job(name="TestJobC")
        testJobGroupA.add(testJobC)
        testJobGroupA.commit()

        result = getAllJobs.execute(subscription=testSubscription["id"])

        assert len(result) == 3, \
            "Wrong number of jobs."

        return

    def testSucceededJobs(self):
        """
        _testSucceededJobs_

        Test the Subscriptions.SucceededJobs DAO turns the correct
        number of succeeded job
        """
        changeJobState = self.daofactory(classname="Jobs.ChangeState")
        getSucceededJobs = self.daofactory(classname="Subscriptions.SucceededJobs")

        testFileset = Fileset(name="TestFileset")
        testFileset.create()

        testWorkflow = Workflow(spec="spec.xml", owner="Hassen",
                                name="wf001", task="Test")

        testWorkflow.create()
        testSubscription = Subscription(fileset=testFileset,
                                        workflow=testWorkflow)
        testSubscription.create()

        testJobGroupA = JobGroup(subscription=testSubscription)
        testJobGroupA.create()

        testJobA = Job(name="TestJobA")
        testJobA.create(testJobGroupA)
        testJobA.changeState("success")
        changeJobState.execute(jobs=[testJobA])
        testJobA["outcome"] = "success"
        testJobA.save()

        testJobB = Job(name="TestJobB")
        testJobB.create(testJobGroupA)
        testJobB.changeState("success")
        changeJobState.execute(jobs=[testJobB])
        testJobB["outcome"] = "success"
        testJobB.save()

        testJobC = Job(name="TestJobC")
        testJobC.create(testJobGroupA)
        testJobC.changeState("jobfailed")
        changeJobState.execute(jobs=[testJobC])
        testJobC["outcome"] = "jobfailed"
        testJobC.save()

        testJobGroupA.add(testJobA)
        testJobGroupA.add(testJobB)
        testJobGroupA.add(testJobC)
        testJobGroupA.commit()

        result = getSucceededJobs.execute(subscription=testSubscription["id"])

        assert len(result) == 2, \
            "Wrong number of jobs."

        testJobD = Job(name="TestJobD")
        testJobD.create(testJobGroupA)
        testJobD.changeState("success")
        changeJobState.execute(jobs=[testJobD])
        testJobD["outcome"] = "success"
        testJobD.save()

        testJobGroupA.add(testJobD)
        testJobGroupA.commit()

        result = getSucceededJobs.execute(subscription=testSubscription["id"])

        assert len(result) == 3, \
            "Wrong number of jobs."

        return

    def testMarkNewFinishedSubscriptions(self):
        """
        _testMarkNewFinishedSubscriptions_

        Verify that the MarkNewFinishedSubscriptions DAO works correctly for
        workflows that don't produce any files.
        """
        testOutputFileset1 = Fileset(name="TestOutputFileset1")
        testOutputFileset1.create()
        testOutputFileset2 = Fileset(name="TestOutputFileset2")
        testOutputFileset2.create()
        testOutputFileset3 = Fileset(name="TestOutputFileset3")
        testOutputFileset3.create()
        testOutputFileset4 = Fileset(name="TestOutputFileset4")
        testOutputFileset4.create()

        testMergedOutputFileset1 = Fileset(name="TestMergedOutputFileset1")
        testMergedOutputFileset1.create()
        testMergedOutputFileset2 = Fileset(name="TestMergedOutputFileset2")
        testMergedOutputFileset2.create()
        testMergedOutputFileset3 = Fileset(name="TestMergedOutputFileset3")
        testMergedOutputFileset3.create()
        testMergedOutputFileset4 = Fileset(name="TestMergedOutputFileset4")
        testMergedOutputFileset4.create()

        testInputFileset = Fileset(name="TestInputFileset")
        testInputFileset.create()
        testInputFileset.markOpen(False)

        testWorkflow1 = Workflow(spec="spec1.xml", owner="Steve",
                                 name="wf001", task="sometask")
        testWorkflow1.create()
        testWorkflow1.addOutput("out1", testOutputFileset1, testMergedOutputFileset1)

        testWorkflow2 = Workflow(spec="spec2.xml", owner="Steve",
                                 name="wf002", task="sometask")
        testWorkflow2.create()
        testWorkflow2.addOutput("out2", testOutputFileset2, testMergedOutputFileset2)

        testWorkflow3 = Workflow(spec="spec3.xml", owner="Steve",
                                 name="wf003", task="sometask")
        testWorkflow3.create()
        testWorkflow3.addOutput("out3", testOutputFileset3, testMergedOutputFileset3)

        testWorkflow4 = Workflow(spec="spec4.xml", owner="Steve",
                                 name="wf004", task="sometask")
        testWorkflow4.create()
        testWorkflow4.addOutput("out4", testOutputFileset4, testMergedOutputFileset4)

        testSubscription1 = Subscription(fileset=testInputFileset,
                                         workflow=testWorkflow1)
        testSubscription1.create()
        testSubscription2 = Subscription(fileset=testOutputFileset1,
                                         workflow=testWorkflow2)
        testSubscription2.create()
        testSubscription3 = Subscription(fileset=testOutputFileset2,
                                         workflow=testWorkflow3)
        testSubscription3.create()

        # Throw in a finished subscription
        testSubscription4 = Subscription(fileset=testOutputFileset3,
                                         workflow=testWorkflow4)
        testSubscription4.create()
        testSubscription4.markFinished()

        myThread = threading.currentThread()
        daoFactory = DAOFactory(package="WMCore.WMBS", logger=myThread.logger,
                                dbinterface=myThread.dbi)
        injected = daoFactory(classname="Workflow.MarkInjectedWorkflows")
        injected.execute(names=["wf001", "wf002", "wf003"], injected=True)
        # The first subscription is finished since the input fileset is closed and no jobs are present
        newFinishedDAO = daoFactory(classname="Subscriptions.MarkNewFinishedSubscriptions")
        newFinishedDAO.execute(self.stateID)
        finishedDAO = daoFactory(classname="Subscriptions.GetFinishedSubscriptions")
        finishedSubs = finishedDAO.execute()

        self.assertEqual(len(finishedSubs), 2,
                         "Wrong number of finished subscriptions.")

        self.assertEqual(finishedSubs[0]["id"], testSubscription1["id"],
                         "Wrong subscription id.")

        # Mark all output filesets which are input of another subscription as closed
        # That should make them all candidates for finalization
        testOutputFileset1.markOpen(False)
        testOutputFileset2.markOpen(False)
        testOutputFileset3.markOpen(False)

        newFinishedDAO.execute(self.stateID)
        finishedSubs = finishedDAO.execute()

        self.assertEqual(len(finishedSubs), 4,
                         "Wrong number of finished subscriptions.")

        finishedWorkflowCheckDAO = daoFactory(classname="Subscriptions.CountFinishedSubscriptionsByWorkflow")
        finishedWfs = [testWorkflow1.name, testWorkflow2.name, testWorkflow3.name, testWorkflow4.name]
        result = finishedWorkflowCheckDAO.execute(workflowNames=finishedWfs)
        self.assertEqual(len(result), 4)
        count = reduce(lambda x, y: {'finished': (x['finished'] + y['finished']),
                                     'open': (x['open'] + y['open'])}, result)
        self.assertEqual(count['finished'], 4)
        self.assertEqual(count['open'], 0)
        return

    def testMarkNewFinishedSubscriptionsTimeout(self):
        """
        _testMarkNewFinishedSubscriptionsTimeout_

        Verify that the finished subscriptions timeout works correctly and that
        it only returns subscriptions for workflows that are fully injected.
        """
        (testSubscription, testFileset, dummyWorkflow,
         testFileA, testFileB, testFileC) = self.createSubscriptionWithFileABC()

        testFileset.markOpen(False)
        testSubscription.create()
        testSubscription.completeFiles([testFileA, testFileB, testFileC])
        testJobGroup = JobGroup(subscription=testSubscription)
        testJobGroup.create()

        testJobA = Job(name="testA")
        testJobA.addFile(testFileA)
        testJobA["location"] = "site1"
        testJobA.create(testJobGroup)
        testJobA["state"] = "complete"

        changeJobState = self.daofactory(classname="Jobs.ChangeState")
        changeJobState.execute([testJobA])

        newFinishedDAO = self.daofactory(classname="Subscriptions.MarkNewFinishedSubscriptions")
        finishedDAO = self.daofactory(classname="Subscriptions.GetFinishedSubscriptions")
        newFinishedDAO.execute(self.stateID)
        finishedSubs = finishedDAO.execute()

        # First we have a job not in cleanout and an uninjected workflow
        self.assertEqual(len(finishedSubs), 0,
                         "There should be no finished subs.")

        time.sleep(5)

        # Even taking into account the delay, we have a not injected workflow
        newFinishedDAO.execute(self.stateID, timeOut=3)
        finishedSubs = finishedDAO.execute()
        self.assertEqual(len(finishedSubs), 0,
                         "There should be no finished subs.")

        injected = self.daofactory(classname="Workflow.MarkInjectedWorkflows")
        injected.execute(names=["wf001", "wfBOGUS"], injected=True)

        # Without timeout we still have a not-cleanout job
        newFinishedDAO.execute(self.stateID)
        finishedSubs = finishedDAO.execute()
        self.assertEqual(len(finishedSubs), 0,
                         "There should be no finished subs.")

        # Now put the timeout in the mix
        newFinishedDAO.execute(self.stateID, timeOut=3)
        finishedSubs = finishedDAO.execute()

        self.assertEqual(len(finishedSubs), 1,
                         "There should be one finished subs.")
        self.assertEqual(finishedSubs[0]["id"], testSubscription["id"],
                         "Wrong finished subscription.")

        return

    def testMarkNewFinishedSubscriptionsParentage(self):
        """
        _testMarkNewFinishedSubscriptionsTimeout_

        Verify that the finished subscriptions DAO can handle the scenario
        when an input file for a subscription which matches all the criteria
        to be defined as finished, is the parent of a file in another fileset
        from another workflow. Therefore the subscription should not be marked
        as finished yet.
        """

        # Let's get the building blocks
        elements = self.createParentageScenario()

        # We want to "finish" the supscription 1 first
        workflow1 = elements['Workflows'][0]
        injected = self.daofactory(classname="Workflow.MarkInjectedWorkflows")
        injected.execute(names=[workflow1.name], injected=True)

        fileset1 = elements['Filesets'][0]
        fileset1.markOpen(False)

        (fileA, fileB) = elements['Files'][:2]
        subscription1 = elements['Subscriptions'][0]
        subscription1.completeFiles([fileA, fileB])

        newFinishedDAO = self.daofactory(classname="Subscriptions.MarkNewFinishedSubscriptions")
        finishedDAO = self.daofactory(classname="Subscriptions.GetFinishedSubscriptions")
        deletableWorkflowDAO = self.daofactory(classname="Workflow.GetDeletableWorkflows")
        newFinishedDAO.execute(self.stateID)
        finishedSubs = finishedDAO.execute()

        self.assertEqual(len(finishedSubs), 1,
                         "There should be one finished subs. but %s found" % len(finishedSubs))

        # Marking the subscription 1 as finished would trigger the deletion of
        # file B which is an error since it is the parent of C.
        dwf = deletableWorkflowDAO.execute()
        self.assertEqual(len(dwf), 0,
                         "There should be no deletable workflow. but %s found" % len(dwf))

        # Now let's finish the second subscription
        workflow2 = elements['Workflows'][1]
        injected.execute(names=[workflow2.name], injected=True)

        fileset2 = elements['Filesets'][1]
        fileset2.markOpen(False)

        fileC = elements['Files'][2]
        subscription2 = elements['Subscriptions'][1]
        subscription2.completeFiles([fileC])

        # Now both workflow is finished
        newFinishedDAO.execute(self.stateID)
        finishedSubs = finishedDAO.execute()

        dwf = deletableWorkflowDAO.execute()
        self.assertEqual(len(dwf), 2,
                         "There should be two deletable workflow. but %s found" % len(dwf))

        self.assertEqual(len(finishedSubs), 2,
                         "There should be two finished sub.")
        self.assertEqual(finishedSubs[1]['id'], subscription2['id'],
                         "The finished sub is not the right one")

        # After another pass we pick up the first subscription
        newFinishedDAO.execute(self.stateID)
        finishedSubs = finishedDAO.execute()

        self.assertEqual(len(finishedSubs), 2,
                         "There should be two finished subs.")

    def testWhitelistBlacklist(self):
        """
        _testWhitelistBlacklist_

        Verify that the white list and black list code works.
        """
        (testSubscription, dummyTestFileset, dummyTestWorkflow,
         dummyTestFileA, dummyTestFileB, dummyTestFileC) = self.createSubscriptionWithFileABC()

        testSubscription.create()
        testSubscription.addWhiteBlackList([{"site_name": "site1", "valid": True},
                                            {"site_name": "site2", "valid": True},
                                            {"site_name": "site3", "valid": False}])
        # try to add duplicate entry : should be ignored (shouldn't throw exception)
        testSubscription.addWhiteBlackList([{"site_name": "site1", "valid": True}]),

        results = testSubscription.getWhiteBlackList()

        self.assertEqual(len(results), 3,
                         "Wrong number of items returned")
        for result in results:
            if result["site_name"] == "site1" or result["site_name"] == "site2":
                self.assertTrue(result["valid"], "Valid should be True.")
            else:
                self.assertFalse(result["valid"], "Valid should be False.")
        return

    def testSubTypesInsertion(self):
        """
        _testSubTypesInsertion_

        Test whether or not we can insert new sub types
        """

        testWorkflow = Workflow(spec="spec.xml", owner="Simon",
                                name="wf001", task="Test")
        testWorkflow.create()

        testFileset = Fileset(name="TestFileset")
        testFileset.create()

        testSubscription = Subscription(fileset=testFileset,
                                        workflow=testWorkflow,
                                        type="newType")

        testSubscription.create()

        test2 = Subscription(id=1)
        test2.load()
        self.assertEqual(test2['type'], 'newType')

        return


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
"""
_Status_

MySQL implementation of Workflow.Status
"""




from WMCore.Database.DBFormatter import DBFormatter

class Status(DBFormatter):
    """
    _Status_

    """
    sql = """SELECT wmbs_users.cert_dn as owner, wmbs_workflow.task, wmbs_job_state.name,
                    COUNT(wmbs_job.id) AS jobs, SUM(wmbs_job.outcome) AS success,
                    SUM(wmbs_fileset.open) AS open FROM wmbs_workflow
               INNER JOIN wmbs_users ON
                 wmbs_users.id = wmbs_workflow.owner
               INNER JOIN wmbs_subscription ON
                 wmbs_workflow.id = wmbs_subscription.workflow
               INNER JOIN wmbs_fileset ON
                 wmbs_subscription.fileset = wmbs_fileset.id
               LEFT OUTER JOIN wmbs_jobgroup ON
                 wmbs_subscription.id = wmbs_jobgroup.subscription
               LEFT OUTER JOIN wmbs_job ON
                 wmbs_jobgroup.id = wmbs_job.jobgroup
               LEFT OUTER JOIN wmbs_job_state ON
                 wmbs_job.state = wmbs_job_state.id
               GROUP BY wmbs_users.id, wmbs_workflow.task, wmbs_job_state.name"""

    def converDecimalToInt(self, results):
        for result in results:
            if result['open'] != None:
                result['open'] = int(result['open'])
            if result['success'] != None:
                result['success'] = int(result['success'])
        return results

    def execute(self, conn = None, transaction = False):
        results = self.dbi.processData(self.sql, conn = conn, transaction = transaction)
        results = self.formatDict(results)
        self.converDecimalToInt(results)
        return results



"""
_StdBase_t_

Created on Jun 14, 2013

@author: dballest
"""
from __future__ import print_function

import json
import unittest

from WMCore.WMSpec.StdSpecs.StdBase import StdBase
from WMCore.WMSpec.WMSpecErrors import WMSpecFactoryException


class StdBaseTest(unittest.TestCase):
    """
    _StdBaseTest_

    A test class for StdBase, it includes tests for
    basic validation functionality.
    """

    def setUp(self):
        """
        _setUp

        Nothing to do
        """
        pass

    def tearDown(self):
        """
        _tearDown_

        Nothing to do
        """
        pass

    def testStdBaseValidation(self):
        """
        _testStdBaseValidation_

        Check that the test arguments pass basic validation,
        i.e. no exception should be raised.
        """
        arguments = StdBase.getTestArguments()
        for k in sorted(arguments.keys()):
            print(k, arguments[k])
        stdBaseInstance = StdBase()
        stdBaseInstance.factoryWorkloadConstruction("TestWorkload", arguments)
        return

    def testRunlist(self):
        """
        _testStdBaseValidation_

        Check that the properly setup run white/black lists
        """
        arguments = StdBase.getTestArguments()
        for k in sorted(arguments.keys()):
            print(k, arguments[k])
        arguments['RunWhiteList'] = ["111111", "222222"]
        stdBaseInstance = StdBase()
        with self.assertRaises(WMSpecFactoryException):
            stdBaseInstance.factoryWorkloadConstruction("TestWorkload", arguments)

    def testCalcEvtsPerJobLumi(self):
        """
        _testCalcEvtsPerJobLumi_

        Check that EventsPerJob and EventsPerLumi are properly calculated
        for EventBased job splitting.
        """
        self.assertEqual((123, 123), StdBase.calcEvtsPerJobLumi(123, 345, 1))
        self.assertEqual((123, 123), StdBase.calcEvtsPerJobLumi(123, None, 1))

        self.assertEqual((28800, 100), StdBase.calcEvtsPerJobLumi(None, 100, 1))
        self.assertEqual((600, 100), StdBase.calcEvtsPerJobLumi(None, 100, 50.5))
        self.assertEqual((570, 570), StdBase.calcEvtsPerJobLumi(None, 1000, 50.5))

        self.assertEqual((23040, 23040), StdBase.calcEvtsPerJobLumi(None, None, 1.25))
        self.assertEqual((229, 229), StdBase.calcEvtsPerJobLumi(None, None, 125.5))

        self.assertEqual((23528, 11764), StdBase.calcEvtsPerJobLumi(24000, 11764, 10.157120496967591))
        self.assertEqual((2835, 2835), StdBase.calcEvtsPerJobLumi(None, 11764, 10.157120496967591))

        self.assertEqual((10, 10), StdBase.calcEvtsPerJobLumi(123, 345, 1, requestedEvents=10))
        self.assertEqual((690, 345), StdBase.calcEvtsPerJobLumi(750, 345, 1, requestedEvents=700))
        self.assertEqual((15000, 100), StdBase.calcEvtsPerJobLumi(None, 100, 1, requestedEvents=15000))
        self.assertEqual((15000, 15000), StdBase.calcEvtsPerJobLumi(None, None, 1, requestedEvents=15000))

    def testValidateGPUSettings(self):
        """
        Test the 'validateGPUSettings' StdBase method.
        """
        with self.assertRaises(WMSpecFactoryException):
            StdBase.validateGPUSettings({"RequiresGPU": "optional"})
        with self.assertRaises(WMSpecFactoryException):
            StdBase.validateGPUSettings({"RequiresGPU": "required"})
        with self.assertRaises(WMSpecFactoryException):
            StdBase.validateGPUSettings({"RequiresGPU": "optional", "GPUParams": json.dumps("")})
        with self.assertRaises(WMSpecFactoryException):
            StdBase.validateGPUSettings({"RequiresGPU": "required", "GPUParams": json.dumps(None)})

        # now input that passes the validation
        self.assertTrue(StdBase.validateGPUSettings({"RequiresGPU": "forbidden"}))
        self.assertTrue(StdBase.validateGPUSettings({"RequiresGPU": "optional",
                                                     "GPUParams": json.dumps("blah")}))
        self.assertTrue(StdBase.validateGPUSettings({"RequiresGPU": "required",
                                                     "GPUParams": json.dumps({"Key1": "value1"})}))


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
"""
_DQMUpload_

Template for a DQMUpload Step

"""

from WMCore.WMSpec.Steps.Template import Template
from WMCore.WMSpec.Steps.Template import CoreHelper
from WMCore.WMSpec.ConfigSectionTree import nodeName


class DQMUploadStepHelper(CoreHelper):
    """
    _DQMUploadStepHelper_

    Add API calls and helper methods to the basic WMStepHelper to specialise
    for DQMUpload tasks

    """
    def setProxyFile(self, proxy):
        """
        Sets the name of the proxy file in the input sandbox
        """
        self.data.upload.proxy = proxy

    def setServerURL(self, url):
        """
        Sets the URL of the DQM GUI server.
        """
        self.data.upload.URL = url


class DQMUpload(Template):
    """
    _DQMUpload_

    Tools for creating a template DQMUpload Step

    """

    def install(self, step):
        stepname = nodeName(step)
        step.stepType = "DQMUpload"
        step.section_("upload")
        step.upload.URL = 'https://cmsweb.cern.ch/dqm/dev'
        step.upload.proxy = None

    def helper(self, step):
        """
        _helper_

        Wrap the WMStep provided in the CMSSW helper class that
        includes the ability to add and manipulate the details
        of a CMSSW workflow step

        """
        return DQMUploadStepHelper(step)



#!/usr/bin/env python
"""
_SubprocessAlgos_t_

Test class for basic subprocess Algorithms
"""


import unittest

from WMCore.Algorithms import SubprocessAlgos

class SubprocessAlgoTest(unittest.TestCase):
    """
    Main test body

    """

    def setUp(self):
        """
        Do nothing
        """
        pass

    def tearDown(self):
        """
        Do nothing
        """
        pass

    def testA_runCommand(self):
        """
        _runCommand_

        Check and see we can run a basic shell command.
        Also check to make sure we get the exit code back correctly
        """

        # First ls a directory you know will have stuff in it
        stdout, stderr, retcode = SubprocessAlgos.runCommand(cmd = 'ls /tmp/')
        self.assertTrue(len(stdout) > 0)
        self.assertEqual(retcode, 0)

        # Now try this same with the shell off
        stdout, stderr, retcode = SubprocessAlgos.runCommand(cmd = ['ls', '/tmp/'])
        self.assertTrue(len(stdout) > 0)
        self.assertEqual(retcode, 0)

        # Now test and see if we can catch a non-zero return code and output
        stdout, stderr, retcode = SubprocessAlgos.runCommand(cmd = 'echo HELP; exit 5')
        self.assertEqual(stdout, 'HELP\n')
        self.assertEqual(stderr, '')
        self.assertEqual(retcode, 5)

        # Now see if we can do a timeout if the process takes too long
        self.assertRaises(SubprocessAlgos.SubprocessAlgoException,
                          SubprocessAlgos.runCommand, cmd = 'sleep 10', timeout = 1)

        # And this one will go on for too long, but not raise because waitTime is not an int
        SubprocessAlgos.runCommand(cmd = 'sleep 1', timeout = 0.1)
        return


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
"""
_ExistsForAccountant_

MySQL implementation of Files.ExistsForAccountant
"""

__all__ = []



from WMCore.Database.DBFormatter import DBFormatter

class ExistsForAccountant(DBFormatter):
    """
    This is highly specialized.  You shouldn't confuse it with
    a normal Exists DAO


    """
    sql = "SELECT lfn FROM dbsbuffer_file WHERE lfn = :lfn"



    def getBinds(self, lfn=None):
        return self.dbi.buildbinds(self.dbi.makelist(lfn), "lfn")

    def execute(self, lfn=None, conn = None, transaction = False):
        result = self.dbi.processData(self.sql, self.getBinds(lfn),
                         conn = conn, transaction = transaction)
        return self.format(result)



"""
Unit tests for MicorService/MSUnmerged/MSUnmergedRSE.py module

"""
from __future__ import division, print_function

import json
import os
import unittest
import mongomock

from pymongo import IndexModel

from WMCore.MicroService.MSUnmerged.MSUnmerged import MSUnmerged, MSUnmergedRSE


class MSUnmergedRSETest(unittest.TestCase):
    """ Unit test for MSUnmergedRSE module """

    def setUp(self):
        """ init test class """

        self.maxDiff = None

        msUnmergedIndex = IndexModel('name', unique=True)
        self.msUnmergedColl = mongomock.MongoClient().msUnmergedDB.msUnmergedColl
        self.msUnmergedColl.create_indexes([msUnmergedIndex])

        self.expectedRSE = {'name': 'T2_US_Wisconsin',
                            'pfnPrefix': None,
                            'isClean': False,
                            "rucioConMonStatus": None,
                            'timestamps': {'rseConsStatTime': 0.0,
                                           'prevStartTime': 0.0,
                                           'startTime': 0.0,
                                           'prevEndTime': 0.0,
                                           'endTime': 0.0},
                            'counters': {'totalNumFiles': 0,
                                         'totalNumDirs': 0,
                                         'dirsToDelete': 0,
                                         'filesToDelete': 0,
                                         'filesDeletedSuccess': 0,
                                         'filesDeletedFail': 0,
                                         'dirsDeletedSuccess': 0,
                                         'dirsDeletedFail': 0,
                                         'gfalErrors': {}},
                            'files': {'allUnmerged': [],
                                      'toDelete': {},
                                      'protected': {},
                                      'deletedSuccess': set(),
                                      'deletedFail': set()},
                            'dirs': {'allUnmerged': set(),
                                     'toDelete': set(),
                                     'protected': set(),
                                     'deletedSuccess': set(),
                                     'deletedFail': set()}}

        super(MSUnmergedRSETest, self).setUp()

    def testCreateRSE(self):
        rse = MSUnmergedRSE('T2_US_Wisconsin')
        self.assertDictEqual(rse, self.expectedRSE)

    def testRSEReadWrite(self):
        rse = MSUnmergedRSE('T2_US_Wisconsin')
        rse.writeRSEToMongoDB(self.msUnmergedColl)
        rse.readRSEFromMongoDB(self.msUnmergedColl)
        rse.pop('_id', None)
        self.assertDictEqual(rse, self.expectedRSE)



#!/usr/bin/env python
"""
_LoadForTaskArchiver_

MySQL implementation of Jobs.LoadForTaskArchiver

Created on Sep 5, 2012

@author: dballest
"""

from WMCore.Database.DBFormatter import DBFormatter

from WMCore.WMBS.Job        import Job
from WMCore.WMBS.File       import File
from WMCore.DataStructs.Run import Run

from future.utils import listvalues

class LoadForTaskArchiver(DBFormatter):
    """
    _LoadForTaskArchiver_

    Retrieve file and mask data for a job given it's ID.
    """

    fileSQL = """SELECT wfd.id, wfd.lfn, wja.job AS jobid
                 FROM wmbs_file_details wfd
                 INNER JOIN wmbs_job_assoc wja ON wja.fileid = wfd.id
                 WHERE wja.job = :jobid"""

    runLumiSQL = """SELECT fileid, run, lumi FROM wmbs_file_runlumi_map
                     WHERE fileid = :fileid"""

    def execute(self, jobID, conn = None, transaction = False):
        """
        _execute_

        Execute the SQL for the given job ID(s) and then format and return
        the result.
        """

        if not isinstance(jobID, list):
            jobID = [jobID]

        binds = [{"jobid": x} for x in jobID]

        if not binds:
            return []

        #First load full file information with run/lumis
        filesResult = self.dbi.processData(self.fileSQL, binds, conn = conn,
                                           transaction = transaction)
        fileList = self.formatDict(filesResult)

        #Clear duplicates
        bindDict = {}
        for result in fileList:
            bindDict[result['id']] = 1
            result['newRuns'] = []
        fileBinds = [{'fileid' : x} for x in bindDict]

        #Load file information
        if len(fileBinds):
            lumiResult = self.dbi.processData(self.runLumiSQL, fileBinds, conn = conn,
                                              transaction = transaction)
            lumiList = self.formatDict(lumiResult)
            lumiDict = {}
            for l in lumiList:
                if not l['fileid'] in lumiDict:
                    lumiDict[l['fileid']] = []
                lumiDict[l['fileid']].append(l)

            for f in fileList:
                fileRuns = {}
                if f['id'] in lumiDict:
                    for l in lumiDict[f['id']]:
                        run = l['run']
                        lumi = l['lumi']
                        try:
                            fileRuns[run].append(lumi)
                        except KeyError:
                            fileRuns[run] = []
                            fileRuns[run].append(lumi)

                for r in fileRuns:
                    newRun = Run(runNumber = r)
                    newRun.lumis = fileRuns[r]
                    f['newRuns'].append(newRun)

        filesForJobs = {}
        for f in fileList:
            jobid = f['jobid']
            if jobid not in filesForJobs:
                filesForJobs[jobid] = {}
            if f['id'] not in filesForJobs[jobid]:
                wmbsFile = File(id = f['id'])
                wmbsFile.update(f)
                for r in wmbsFile['newRuns']:
                    wmbsFile.addRun(r)
                filesForJobs[jobid][f['id']] = wmbsFile


        #Add the file information to job objects and load the masks
        jobList = [Job(id = x) for x in jobID]
        for j in jobList:
            if j['id'] in filesForJobs:
                j['input_files'] = listvalues(filesForJobs[j['id']])
            j['mask'].load(j['id'])

        return jobList



from __future__ import division

import unittest
from WMCore.BossAir.Plugins.MockPlugin import MockPlugin
from WMCore.BossAir.Plugins.BasePlugin import BossAirPluginException
from WMCore.Configuration import Configuration
from WMQuality.TestInit import TestInit
from datetime import datetime
from datetime import timedelta
import os


TEST_JOB_LEN = 60
config = Configuration()
config.section_('BossAir')
config.BossAir.section_('MockPlugin')
config.BossAir.MockPlugin.jobRunTime = TEST_JOB_LEN
config.BossAir.MockPlugin.fakeReport = os.path.join(os.path.dirname(__file__), 'FakeReport.pkl')
config.BossAir.MockPlugin.fakeReport = os.path.join(os.path.dirname(__file__), 'LogCollectFakeReport.pkl')


jobList = [{'status': 'Done', 'bulkid': None, 'cms_site_name': None, 'cache_dir': '/data/wmagent/osb/JobCache/amaltaro_whatever_110609_114309/Production/JobCollection_23_0/job_46', 'taskType': None, 'id': 1, 'plugin': 'MockPlugin', 'gridid': None, 'userdn': '/DC=ch/DC=cern/OU=Organic Units/OU=Users/CN=mmascher/CN=720897/CN=Marco Mascheroni', 'jobid': 46, 'priority': None, 'retry_count': 0, 'sandbox': None, 'globalState': 'Running', 'location': None, 'packageDir': None, 'status_time': None}]



class MockPluginTest(unittest.TestCase):
    def setUp(self):
        self.testinit = TestInit(__file__)
        self.workdir = self.testinit.generateWorkDir()
        jobList[0]['cache_dir'] = self.workdir

    def tearDown(self):
        self.testinit.delWorkDir()

    def testInit(self):
        wrongconfig = Configuration()
        wrongconfig.section_('BossAir')
        self.assertRaises( BossAirPluginException, MockPlugin, wrongconfig )

        wrongconfig.BossAir.section_('MockPlugin')
        self.assertRaises( BossAirPluginException, MockPlugin, wrongconfig )
        #The config does not contain fakeReport parameter
        self.assertRaises( BossAirPluginException, MockPlugin, wrongconfig )

        #The fakeReport does not exist
        wrongconfig.BossAir.MockPlugin.fakeReport = 'asdf'
        self.assertRaises( BossAirPluginException, MockPlugin, wrongconfig )

    def testTrack(self):
        mp = MockPlugin(config)

        #Check that the job has been scheduled
        self.assertEqual({}, mp.jobsScheduledEnd)

        # Don't be racy
        currentTime = datetime.now()
        #id is the only required parameter in the job dictionary
        res = mp.track( jobList, currentTime )
        self.assertTrue( 1 in mp.jobsScheduledEnd )
        #check scheduled end (N.B. this includes 20% of random time)
        scheduledEnd = mp.jobsScheduledEnd[1]
        timeTillJob = scheduledEnd - currentTime
        self.assertTrue( timeTillJob >= timedelta(minutes = TEST_JOB_LEN - 1), \
                         "Time till Job %s !>= Delta %s" % (timeTillJob, \
                         timedelta(minutes = TEST_JOB_LEN - 1)))
        self.assertTrue( timeTillJob <= timedelta(minutes = TEST_JOB_LEN*120/100 + 1), \
                         "Time till Job %s !<= Delta %s" % (timeTillJob, \
                         timedelta(minutes = TEST_JOB_LEN * 120/100 + 1)) )
        #the job is running
        self.assertEqual( 'Running', res[0][0]['status'])
        self.assertEqual( 'Running', res[1][0]['status'])
        self.assertEqual( [], res[2])

        #the job is not running anymore
        mp.jobsScheduledEnd[1] = datetime(1900,1,1)
        res = mp.track( jobList )
        self.assertEqual( [], res[0])
        self.assertEqual( 'Done', res[1][0]['status'])
        self.assertEqual( 'Done', res[2][0]['status'])

        del mp



if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
_AlgoDatasetAssoc_

Associate an algorithm with a dataset in DBSBuffer.
"""

from WMComponent.DBS3Buffer.MySQL.AlgoDatasetAssoc import AlgoDatasetAssoc as MySQLAlgoDatasetAssoc

class AlgoDatasetAssoc(MySQLAlgoDatasetAssoc):

    sql = """INSERT INTO dbsbuffer_algo_dataset_assoc
             (id, algo_id, dataset_id)
             SELECT dbsbuffer_algdset_assoc_seq.nextval,
                    (SELECT id FROM dbsbuffer_algo
                     WHERE app_name = :app_name AND app_ver = :app_ver
                     AND app_fam = :app_fam AND pset_hash = :pset_hash),
                    (SELECT id FROM dbsbuffer_dataset WHERE path = :path)
             FROM DUAL
             WHERE NOT EXISTS
               ( SELECT *
                 FROM dbsbuffer_algo_dataset_assoc
                 WHERE algo_id =
                   (SELECT id FROM dbsbuffer_algo
                    WHERE app_name = :app_name AND app_ver = :app_ver
                    AND app_fam = :app_fam AND pset_hash = :pset_hash)
                 AND dataset_id =
                   (SELECT id FROM dbsbuffer_dataset WHERE path = :path) )
             """



import logging
from functools import wraps

from WMCore.Configuration import Configuration
from WMCore.WebTools.Root import Root


#this function will be used for cherrypy set up for test
def cherrypySetup(config = None):
    if config == None:
        config = DefaultConfig()
    def chSetup(f):
        @wraps(f)
        def wrapper(self):
            self.rt = Root(config, testName = f.__name__)
            self.rt.start(blocking=False)
            self.urlbase = config.getServerUrl()
            f(self)
            self.rt.stop()
            self.rt.setLastTest()
        return wrapper

    return chSetup


class DefaultConfig(Configuration):

    def __init__(self, model=None):
        Configuration.__init__(self)
        self.component_('SecurityModule')
        self.SecurityModule.dangerously_insecure = True

        self.component_('Webtools')
        self.Webtools.application = 'UnitTests'
        self.Webtools.log_screen = False
        self.Webtools.environment = "development"
        self.Webtools.error_log_level = logging.DEBUG
        self.Webtools.access_log_level = logging.DEBUG
        self.Webtools.port = 8888
        self.Webtools.host = "localhost"
        self.Webtools.expires = 300
        self.Webtools.thread_pool = 10  # CherryPy default
        self.Webtools.accepted_queue_size = -1  # CherryPy default
        self.Webtools.accepted_queue_timeout = 10  # CherryPy default

        self.component_('UnitTests')
        self.UnitTests.title = 'CMS WMCore/WebTools Unit Tests'
        self.UnitTests.description = 'Dummy server for the running of unit tests'
        self.UnitTests.admin ="UnitTestAdmin"
        self.UnitTests.templates = "/tmp"
        self.UnitTests.section_('views')

        active = self.UnitTests.views.section_('active')
        active.section_('rest')
        active.rest.application = 'UnitTestRESTApp'
        active.rest.object = 'WMCore.WebTools.RESTApi'
        active.rest.section_('database')
        active.rest.database.connectUrl = 'sqlite://'
        active.rest.section_('model')
        active.rest.model.object = model or 'WMCore.WebTools.RESTModel'
        active.rest.section_('formatter')
        active.rest.formatter.object = 'WMCore.WebTools.RESTFormatter'
        active.rest.formatter.templates = '/tmp'
        #WARNING: need is not actual config - if cherrypy is started by Root.py
        #This will handled automatically - added here just for DummyModel test.
        active.rest.default_expires = self.Webtools.expires

    def getServerUrl(self):
        return "http://%s:%s/%s/rest/" % (self.Webtools.host, self.Webtools.port, self.Webtools.application.lower())

    def getDBUrl(self):
        return self.UnitTests.views.active.rest.database.connectUrl

    def setDBUrl(self, dbUrl):
        self.UnitTests.views.active.rest.database.connectUrl = dbUrl

    def setPort(self, port):
        self.Webtools.port = port

    def setFormatter(self, formatter):
        self.UnitTests.views.active.rest.formatter.object = formatter

    def getModelConfig(self):
        return self.UnitTests.views.active.rest
